from crispy_forms.helper import FormHelper
from crispy_forms_foundation.layout import (
    ButtonHolder,
    Column,
    Fieldset,
    Layout,
    Row,
    Submit,
)
from dal import autocomplete
from django import forms
from django.core.exceptions import ValidationError
from django.forms import RadioSelect

from . import const
from .models.uczelnia import RaportSlotowUczelnia

from bpp.const import PBN_MAX_ROK, PBN_MIN_ROK
from bpp.models import Autor, Uczelnia
from bpp.util import formdefaults_html_after, formdefaults_html_before, year_last_month

OUTPUT_FORMATS = [
    ("html", "wyświetl w przeglądarce"),
    ("xlsx", "Microsoft Excel (XLSX)"),
]

OUTPUT_FORMATS_WITH_PDF = OUTPUT_FORMATS + [
    ("pdf", "Portable Document Format (PDF)"),
]


class AutorRaportSlotowForm(forms.Form):
    obiekt = forms.ModelChoiceField(
        label="Autor",
        queryset=Autor.objects.all(),
        widget=autocomplete.ModelSelect2(url="bpp:public-autor-autocomplete"),
    )

    od_roku = forms.IntegerField(initial=year_last_month, min_value=2016)
    do_roku = forms.IntegerField(initial=Uczelnia.objects.do_roku_default)

    minimalny_pk = forms.IntegerField(label="Minimalna wartość PK pracy", initial=0)

    dzialanie = forms.ChoiceField(
        label="Wygeneruj",
        choices=(
            (
                const.DZIALANIE_WSZYSTKO,
                "prace autora z punktacją dla dyscyplin za dany okres",
            ),
            (const.DZIALANIE_SLOT, "zbierz najlepsze prace do zadanej wielkości slotu"),
        ),
        initial="wszystko",
        widget=forms.RadioSelect,
    )

    slot = forms.DecimalField(
        label="Zadana wielkość slotu",
        required=False,
        max_digits=8,
        decimal_places=4,
        max_value=20,
    )

    _export = forms.ChoiceField(
        label="Format wyjściowy", choices=OUTPUT_FORMATS_WITH_PDF, required=True
    )

    def clean(self):
        if "od_roku" in self.cleaned_data and "do_roku" in self.cleaned_data:
            if self.cleaned_data["od_roku"] > self.cleaned_data["do_roku"]:
                raise ValidationError(
                    {
                        "od_roku": ValidationError(
                            'Pole musi być większe lub równe jak pole "Do roku".',
                            code="od_do_zle",
                        )
                    }
                )

        if (
            self.cleaned_data["dzialanie"] == const.DZIALANIE_WSZYSTKO
            and "slot" in self.cleaned_data
            and self.cleaned_data["slot"] is not None
        ):
            raise ValidationError(
                {
                    "slot": ValidationError(
                        "Gdy chcesz wygenerować wszystkie prace tego autora, pozostaw pole 'Slot' puste. ",
                        code="nie_podawaj_gdy_wszystko",
                    )
                }
            )

        if self.cleaned_data["dzialanie"] == const.DZIALANIE_SLOT and (
            "slot" not in self.cleaned_data
            or ("slot" in self.cleaned_data and self.cleaned_data["slot"] is None)
            or (
                "slot" in self.cleaned_data
                and self.cleaned_data["slot"] is not None
                and self.cleaned_data["slot"] <= 0
            )
        ):
            raise ValidationError(
                {
                    "slot": ValidationError(
                        "Podaj wartość slota do którego chcesz zbierać prace. Wartość musi być większa od zera. ",
                        code="podawaj_gdy_slot",
                    )
                }
            )

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.helper = FormHelper()
        self.helper.form_class = "custom"
        self.helper.form_action = "."
        self.helper.layout = Layout(
            Fieldset(
                "Wybierz parametry",
                formdefaults_html_before(self),
                Row(Column("obiekt", css_class="large-12 small-12")),
                Row(Column("dzialanie", css_class="large-12 small-12")),
                Row(Column("slot", css_class="large-12 small-12")),
                Row(
                    Column("od_roku", css_class="large-6 small-6"),
                    Column("do_roku", css_class="large-6 small-6"),
                ),
                Row(Column("minimalny_pk")),
                Row(Column("_export")),
                formdefaults_html_after(self),
            ),
            ButtonHolder(
                Submit(
                    "submit",
                    "Pobierz raport",
                    css_id="id_submit",
                    css_class="submit button",
                ),
            ),
        )


class UtworzRaportSlotowUczelniaForm(forms.ModelForm):
    class Meta:
        model = RaportSlotowUczelnia
        fields = [
            "od_roku",
            "do_roku",
            "akcja",
            "slot",
            "minimalny_pk",
            "dziel_na_jednostki_i_wydzialy",
            "pokazuj_zerowych",
        ]

    def __init__(self, *args, **kwargs):
        self.helper = FormHelper()
        self.helper.form_class = "custom"
        self.helper.form_action = "."
        self.helper.layout = Layout(
            Fieldset(
                "Wybierz parametry",
                formdefaults_html_before(self),
                Row(
                    Column("od_roku", css_class="large-6 small-6"),
                    Column("do_roku", css_class="large-6 small-6"),
                ),
                Row(Column("akcja", css_class="large-12 small-12")),
                Row(Column("slot", css_class="large-12 small-12")),
                Row(Column("minimalny_pk", css_class="large-12 small-12")),
                Row(Column("dziel_na_jednostki_i_wydzialy")),
                Row(Column("pokazuj_zerowych")),
                formdefaults_html_after(self),
            ),
            ButtonHolder(
                Submit(
                    "submit",
                    "Utwórz raport",
                    css_id="id_submit",
                    css_class="submit button",
                ),
            ),
        )

        super().__init__(*args, **kwargs)


class ParametryRaportSlotowEwaluacjaForm(forms.Form):
    od_roku = forms.IntegerField(
        initial=Uczelnia.objects.do_roku_default, min_value=PBN_MIN_ROK
    )
    do_roku = forms.IntegerField(
        initial=Uczelnia.objects.do_roku_default,
        min_value=PBN_MIN_ROK,
        max_value=PBN_MAX_ROK,
    )

    _export = forms.ChoiceField(
        label="Format wyjściowy",
        choices=OUTPUT_FORMATS,
        required=True,
        widget=RadioSelect,
        initial="html",
    )
    upowaznienie_pbn = forms.NullBooleanField(
        required=False,
        # widget=RadioSelect,
    )

    def clean(self):
        if "od_roku" in self.cleaned_data and "do_roku" in self.cleaned_data:
            if self.cleaned_data["od_roku"] > self.cleaned_data["do_roku"]:
                raise ValidationError(
                    {
                        "od_roku": ValidationError(
                            'Pole musi być większe lub równe jak pole "Do roku".',
                            code="od_do_zle",
                        )
                    }
                )

    def __init__(self, *args, **kwargs):
        self.helper = FormHelper()
        self.helper.form_class = "custom"
        self.helper.form_action = "."
        self.helper.layout = Layout(
            Fieldset(
                "Wybierz parametry",
                formdefaults_html_before(self),
                Row(
                    Column("od_roku", css_class="large-6 small-6"),
                    Column("do_roku", css_class="large-6 small-6"),
                ),
                Row(Column("upowaznienie_pbn", css_class="large-12 small-12")),
                Row(Column("_export")),
                formdefaults_html_after(self),
            ),
            ButtonHolder(
                Submit(
                    "submit",
                    "Pobierz raport",
                    css_id="id_submit",
                    css_class="submit button",
                ),
            ),
        )

        super().__init__(*args, **kwargs)



"""Backport klasy z customizowalnymi parametrami formsetu czyli
ModelAdmin.get_formset_kwargs

"""
import django


class CustomizableFormsetParamsAdminMixinWyrzucWDjango40:
    """Mixin zapewniający funkcję ``get_formset_kwargs`` z Django 4.0,
    można go usunąć po przejściu na tą wersję.

    https://github.com/django/django/commit/3119a6decab7788eca662b10e8c18351d20df212
    """

    def get_formset_kwargs(self, request, obj, inline, prefix):
        formset_params = {
            "instance": obj,
            "prefix": prefix,
            "queryset": inline.get_queryset(request),
        }
        if request.method == "POST":
            formset_params.update(
                {
                    "data": request.POST.copy(),
                    "files": request.FILES,
                    "save_as_new": "_saveasnew" in request.POST,
                }
            )
        return formset_params

    def _create_formsets(self, request, obj, change):
        "Helper function to generate formsets for add/change_view."

        class AlreadyImplementedError(NotImplementedError):
            pass

        if django.VERSION >= (4, 0):
            # Nie wymagamy tego w Django 4.0
            raise AlreadyImplementedError(
                "Uzywasz Django 4.0, wyrzuc ten mixin i uzyj natywnej funkcji"
            )

        formsets = []
        inline_instances = []
        prefixes = {}
        get_formsets_args = [request]
        if change:
            get_formsets_args.append(obj)
        for FormSet, inline in self.get_formsets_with_inlines(*get_formsets_args):
            prefix = FormSet.get_default_prefix()
            prefixes[prefix] = prefixes.get(prefix, 0) + 1
            if prefixes[prefix] != 1 or not prefix:
                prefix = f"{prefix}-{prefixes[prefix]}"
            formset_params = self.get_formset_kwargs(request, obj, inline, prefix)
            formset = FormSet(**formset_params)

            def user_deleted_form(request, obj, formset, index, inline=inline):
                """Return whether or not the user deleted the form."""
                return (
                    inline.has_delete_permission(request, obj)
                    and f"{formset.prefix}-{index}-DELETE" in request.POST
                )

            # Bypass validation of each view-only inline form (since the form's
            # data won't be in request.POST), unless the form was deleted.
            if not inline.has_change_permission(request, obj if change else None):
                for index, form in enumerate(formset.initial_forms):
                    if user_deleted_form(request, obj, formset, index):
                        continue
                    form._errors = {}
                    form.cleaned_data = form.initial
            formsets.append(formset)
            inline_instances.append(inline)
        return formsets, inline_instances



# -*- coding: utf-8 -*-


from django.db import migrations, models
import django.contrib.postgres.fields


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0072_gist_to_gin'),
    ]

    operations = [
        migrations.AlterField(
            model_name='patent',
            name='opis_bibliograficzny_autorzy_cache',
            field=django.contrib.postgres.fields.ArrayField(size=None, null=True, base_field=models.TextField(), blank=True),
        ),
        migrations.AlterField(
            model_name='praca_doktorska',
            name='opis_bibliograficzny_autorzy_cache',
            field=django.contrib.postgres.fields.ArrayField(size=None, null=True, base_field=models.TextField(), blank=True),
        ),
        migrations.AlterField(
            model_name='praca_habilitacyjna',
            name='opis_bibliograficzny_autorzy_cache',
            field=django.contrib.postgres.fields.ArrayField(size=None, null=True, base_field=models.TextField(), blank=True),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='opis_bibliograficzny_autorzy_cache',
            field=django.contrib.postgres.fields.ArrayField(size=None, null=True, base_field=models.TextField(), blank=True),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='opis_bibliograficzny_autorzy_cache',
            field=django.contrib.postgres.fields.ArrayField(size=None, null=True, base_field=models.TextField(), blank=True),
        ),
    ]



# Generated by Django 3.2.15 on 2022-08-17 22:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("zglos_publikacje", "0009_alter_zgloszenie_publikacji_status"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="zgloszenie_publikacji",
            options={
                "ordering": ("-ostatnio_zmieniony", "tytul_oryginalny"),
                "verbose_name": "zgłoszenie publikacji",
                "verbose_name_plural": "zgłoszenia publikacji",
            },
        ),
        migrations.AddField(
            model_name="zgloszenie_publikacji",
            name="rodzaj_zglaszanej_publikacji",
            field=models.PositiveSmallIntegerField(
                choices=[
                    (1, "artykuł naukowy lub monografia"),
                    (2, "pozostałe rodzaje"),
                ],
                default=1,
                help_text="Dla artykułów naukowych i monografii konieczne będzie wprowadzenie "
                "informacji o kosztach w ostatnim etapie wypełniania formularza. ",
                verbose_name="Rodzaj zgłaszanej publikacji",
            ),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name="zgloszenie_publikacji",
            name="kod_do_edycji",
            field=models.UUIDField(blank=True, editable=False, null=True, unique=True),
        ),
    ]



from rest_framework import serializers
from taggit_serializer.serializers import TagListSerializerField

from api_v1.serializers.util import (
    AbsoluteUrlSerializerMixin,
    Wydawnictwo_AutorSerializerMixin,
    WydawnictwoSerializerMixin,
)

from bpp.models import (
    Wydawnictwo_Ciagle,
    Wydawnictwo_Ciagle_Autor,
    Wydawnictwo_Ciagle_Streszczenie,
    Wydawnictwo_Ciagle_Zewnetrzna_Baza_Danych,
)


class Wydawnictwo_Ciagle_Zewnetrzna_Baza_DanychSerializer(serializers.ModelSerializer):
    rekord = serializers.HyperlinkedRelatedField(
        view_name="api_v1:wydawnictwo_ciagle-detail", read_only=True
    )
    baza = serializers.StringRelatedField()

    class Meta:
        model = Wydawnictwo_Ciagle_Zewnetrzna_Baza_Danych
        fields = ["rekord", "baza", "info"]


class Wydawnictwo_Ciagle_StreszczenieSerializer(serializers.ModelSerializer):
    rekord = serializers.HyperlinkedRelatedField(
        view_name="api_v1:wydawnictwo_ciagle-detail", read_only=True
    )

    class Meta:
        model = Wydawnictwo_Ciagle_Streszczenie
        fields = ["rekord", "jezyk_streszczenia", "streszczenie"]


class Wydawnictwo_Ciagle_AutorSerializer(
    Wydawnictwo_AutorSerializerMixin, serializers.HyperlinkedModelSerializer
):
    rekord = serializers.HyperlinkedRelatedField(
        view_name="api_v1:wydawnictwo_ciagle-detail", read_only=True
    )

    class Meta:
        model = Wydawnictwo_Ciagle_Autor
        fields = [
            "id",
            "autor",
            "jednostka",
            "zapisany_jako",
            "typ_odpowiedzialnosci",
            "afiliuje",
            "zatrudniony",
            "kolejnosc",
            "rekord",
            "procent",
            "dyscyplina_naukowa",
        ]


class Wydawnictwo_CiagleSerializer(
    AbsoluteUrlSerializerMixin,
    WydawnictwoSerializerMixin,
    serializers.HyperlinkedModelSerializer,
):

    slowa_kluczowe = TagListSerializerField()

    zrodlo = serializers.HyperlinkedRelatedField(
        view_name="api_v1:zrodlo-detail", read_only=True
    )

    autorzy_set = serializers.HyperlinkedRelatedField(
        many=True, view_name="api_v1:wydawnictwo_ciagle_autor-detail", read_only=True
    )

    konferencja = serializers.HyperlinkedRelatedField(
        view_name="api_v1:konferencja-detail", read_only=True
    )

    zewnetrzna_baza_danych = serializers.HyperlinkedRelatedField(
        many=True,
        view_name="api_v1:wydawnictwo_ciagle_zewnetrzna_baza_danych-detail",
        read_only=True,
    )

    streszczenia = serializers.HyperlinkedRelatedField(
        many=True,
        view_name="api_v1:wydawnictwo_ciagle_streszczenie-detail",
        read_only=True,
    )

    class Meta:
        model = Wydawnictwo_Ciagle
        fields = [
            "id",
            #
            "tytul_oryginalny",
            "tytul",
            #
            "rok",
            "status_korekty",
            #
            "jezyk",
            "jezyk_alt",
            "charakter_formalny",
            "typ_kbn",
            #
            "zrodlo",
            #
            "www",
            "dostep_dnia",
            "public_www",
            "public_dostep_dnia",
            #
            "pubmed_id",
            "pmc_id",
            "doi",
            "pbn_id",
            #
            "praca_wybitna",
            "uzasadnienie_wybitnosci",
            #
            "nr_zeszytu",
            #
            "konferencja",
            #
            "impact_factor",
            "punkty_kbn",
            #
            "informacje",
            "szczegoly",
            "uwagi",
            "slowa_kluczowe",
            "strony",
            "tom",
            #
            "issn",
            "e_issn",
            #
            "liczba_cytowan",
            #
            "openaccess_tryb_dostepu",
            "openaccess_wersja_tekstu",
            "openaccess_licencja",
            "openaccess_czas_publikacji",
            "openaccess_ilosc_miesiecy",
            #
            "utworzono",
            "ostatnio_zmieniony",
            #
            "absolute_url",
            #
            "tekst_przed_pierwszym_autorem",
            "autorzy_set",
            "tekst_po_ostatnim_autorze",
            #
            "zewnetrzna_baza_danych",
            "streszczenia",
        ]



from urllib.parse import parse_qs
from urllib.parse import quote as urlquote

from django import forms
from django.db import models
from django.db.models import Q
from django.forms import BaseInlineFormSet
from django.forms.widgets import Textarea
from django.urls import reverse

from import_common.normalization import normalize_isbn
from pbn_api.exceptions import (
    AccessDeniedException,
    NeedsPBNAuthorisationException,
    PKZeroExportDisabled,
    SameDataUploadedRecently,
)
from pbn_api.models import SentData

from django.contrib import messages
from django.contrib.admin.utils import quote
from django.contrib.contenttypes.models import ContentType

from django.utils.html import format_html
from django.utils.safestring import mark_safe

from bpp import const
from bpp.const import PBN_MAX_ROK, PBN_MIN_ROK
from bpp.models import Status_Korekty
from bpp.models.sloty.core import ISlot
from bpp.models.sloty.exceptions import CannotAdapt

CHARMAP_SINGLE_LINE = forms.TextInput(
    attrs={"class": "charmap", "style": "width: 500px"}
)


# Pomocnik dla klasy ModelZMetryczka


class ZapiszZAdnotacjaMixin:
    readonly_fields = ("ostatnio_zmieniony",)


class AdnotacjeZDatamiMixin:
    readonly_fields = ("utworzono", "ostatnio_zmieniony", "id")


class AdnotacjeZDatamiOrazPBNMixin:
    readonly_fields = (
        "utworzono",
        "ostatnio_zmieniony",
        "id",
        "pbn_id",
    )


ADNOTACJE_FIELDSET = (
    "Adnotacje",
    {
        "classes": ("grp-collapse grp-closed",),
        "fields": (ZapiszZAdnotacjaMixin.readonly_fields + ("adnotacje",)),
    },
)

ADNOTACJE_Z_DATAMI_FIELDSET = (
    "Adnotacje",
    {
        "classes": ("grp-collapse grp-closed",),
        "fields": AdnotacjeZDatamiMixin.readonly_fields + ("adnotacje",),
    },
)

ADNOTACJE_Z_DATAMI_ORAZ_PBN_FIELDSET = (
    "Adnotacje",
    {
        "classes": ("grp-collapse grp-closed",),
        "fields": AdnotacjeZDatamiOrazPBNMixin.readonly_fields + ("adnotacje",),
    },
)

OPENACCESS_FIELDSET = (
    "OpenAccess",
    {
        "classes": ("grp-collapse grp-closed",),
        "fields": (
            "openaccess_tryb_dostepu",
            "openaccess_licencja",
            "openaccess_wersja_tekstu",
            "openaccess_czas_publikacji",
            "openaccess_ilosc_miesiecy",
        ),
    },
)

DWA_TYTULY = (
    "tytul_oryginalny",
    "tytul",
)

MODEL_ZE_SZCZEGOLAMI = (
    "informacje",
    "szczegoly",
    "uwagi",
    "slowa_kluczowe",
    "strony",
    "tom",
)

MODEL_Z_ISSN = (
    "issn",
    "e_issn",
)

MODEL_Z_PBN_UID = ("pbn_uid",)

MODEL_Z_OPLATA_ZA_PUBLIKACJE = (
    "opl_pub_cost_free",
    "opl_pub_research_potential",
    "opl_pub_research_or_development_projects",
    "opl_pub_other",
    "opl_pub_amount",
)

MODEL_Z_OPLATA_ZA_PUBLIKACJE_FIELDSET = (
    "Opłata za publikację",
    {"classes": ("grp-collapse grp-closed",), "fields": MODEL_Z_OPLATA_ZA_PUBLIKACJE},
)


MODEL_Z_ISBN = (
    "isbn",
    "e_isbn",
)

MODEL_Z_WWW = (
    "www",
    "dostep_dnia",
    "public_www",
    "public_dostep_dnia",
)

MODEL_Z_PUBMEDID = ("pubmed_id", "pmc_id")

MODEL_Z_DOI = ("doi",)

MODEL_Z_LICZBA_CYTOWAN = ("liczba_cytowan",)

MODEL_Z_MIEJSCEM_PRZECHOWYWANIA = ("numer_odbitki",)

MODEL_Z_ROKIEM = ("rok",)

MODEL_TYPOWANY = (
    "jezyk",
    "jezyk_alt",
    "jezyk_orig",
    "typ_kbn",
)

MODEL_PUNKTOWANY_BAZA = (
    "punkty_kbn",
    "impact_factor",
    "index_copernicus",
    "punktacja_snip",
    "punktacja_wewnetrzna",
)

MODEL_PUNKTOWANY = MODEL_PUNKTOWANY_BAZA + ("weryfikacja_punktacji",)

MODEL_PUNKTOWANY_Z_KWARTYLAMI_BAZA = MODEL_PUNKTOWANY_BAZA + (
    "kwartyl_w_scopus",
    "kwartyl_w_wos",
)

MODEL_PUNKTOWANY_Z_KWARTYLAMI = MODEL_PUNKTOWANY_Z_KWARTYLAMI_BAZA + (
    "weryfikacja_punktacji",
)

MODEL_PUNKTOWANY_KOMISJA_CENTRALNA = (
    "kc_impact_factor",
    "kc_punkty_kbn",
    "kc_index_copernicus",
)

MODEL_Z_INFORMACJA_Z = ("informacja_z",)

MODEL_Z_LICZBA_ZNAKOW_WYDAWNICZYCH = ("liczba_znakow_wydawniczych",)

MODEL_ZE_STATUSEM = ("status_korekty",)

MODEL_RECENZOWANY = ("recenzowana",)

MODEL_TYPOWANY_BEZ_CHARAKTERU_FIELDSET = (
    "Typ",
    {"classes": ("",), "fields": MODEL_TYPOWANY},
)

MODEL_TYPOWANY_FIELDSET = (
    "Typ",
    {"classes": ("",), "fields": ("charakter_formalny",) + MODEL_TYPOWANY},
)

MODEL_PUNKTOWANY_FIELDSET = (
    "Punktacja",
    {"classes": ("",), "fields": MODEL_PUNKTOWANY},
)

MODEL_PUNKTOWANY_WYDAWNICTWO_CIAGLE_FIELDSET = (
    "Punktacja",
    {
        "classes": ("",),
        "fields": MODEL_PUNKTOWANY_Z_KWARTYLAMI + ("uzupelnij_punktacje",),
    },
)

MODEL_PUNKTOWANY_KOMISJA_CENTRALNA_FIELDSET = (
    "Punktacja Komisji Centralnej",
    {
        "classes": ("grp-collapse grp-closed",),
        "fields": MODEL_PUNKTOWANY_KOMISJA_CENTRALNA,
    },
)

MODEL_OPCJONALNIE_NIE_EKSPORTOWANY_DO_API_FIELDSET = (
    "Eksport do API",
    {"classes": ("grp-collapse grp-closed",), "fields": ("nie_eksportuj_przez_api",)},
)

POZOSTALE_MODELE_FIELDSET = (
    "Pozostałe informacje",
    {
        "classes": ("",),
        "fields": MODEL_Z_INFORMACJA_Z + MODEL_ZE_STATUSEM + MODEL_RECENZOWANY,
    },
)

POZOSTALE_MODELE_WYDAWNICTWO_CIAGLE_FIELDSET = (
    "Pozostałe informacje",
    {
        "classes": ("",),
        "fields": MODEL_Z_LICZBA_ZNAKOW_WYDAWNICZYCH
        + MODEL_Z_INFORMACJA_Z
        + MODEL_ZE_STATUSEM
        + MODEL_RECENZOWANY,
    },
)

POZOSTALE_MODELE_WYDAWNICTWO_ZWARTE_FIELDSET = (
    "Pozostałe informacje",
    {
        "classes": ("",),
        "fields": MODEL_Z_LICZBA_ZNAKOW_WYDAWNICZYCH
        + MODEL_Z_INFORMACJA_Z
        + MODEL_ZE_STATUSEM
        + MODEL_RECENZOWANY,
    },
)

SERIA_WYDAWNICZA_FIELDSET = (
    "Seria wydawnicza",
    {
        "classes": ("grp-collapse grp-closed",),
        "fields": ("seria_wydawnicza", "numer_w_serii"),
    },
)

PRACA_WYBITNA_FIELDSET = (
    "Praca wybitna",
    {
        "classes": ("grp-collapse grp-closed",),
        "fields": ("praca_wybitna", "uzasadnienie_wybitnosci"),
    },
)

PRZED_PO_LISCIE_AUTOROW_FIELDSET = (
    "Dowolny tekst przed lub po liście autorów",
    {
        "classes": ("grp-collapse grp-closed",),
        "fields": ("tekst_przed_pierwszym_autorem", "tekst_po_ostatnim_autorze"),
    },
)

EKSTRA_INFORMACJE_WYDAWNICTWO_CIAGLE_FIELDSET = (
    "Ekstra informacje",
    {
        "classes": ("grp-collapse grp-closed",),
        "fields": MODEL_Z_PBN_UID
        + MODEL_Z_ISSN
        + MODEL_Z_WWW
        + MODEL_Z_PUBMEDID
        + MODEL_Z_DOI
        + MODEL_Z_LICZBA_CYTOWAN
        + MODEL_Z_MIEJSCEM_PRZECHOWYWANIA,
    },
)

EKSTRA_INFORMACJE_WYDAWNICTWO_ZWARTE_FIELDSET = (
    "Ekstra informacje",
    {
        "classes": ("grp-collapse grp-closed",),
        "fields": MODEL_Z_PBN_UID
        + MODEL_Z_ISSN
        + MODEL_Z_WWW
        + MODEL_Z_PUBMEDID
        + MODEL_Z_DOI
        + MODEL_Z_LICZBA_CYTOWAN
        + MODEL_Z_MIEJSCEM_PRZECHOWYWANIA,
    },
)

EKSTRA_INFORMACJE_DOKTORSKA_HABILITACYJNA_FIELDSET = (
    "Ekstra informacje",
    {
        "classes": ("grp-collapse grp-closed",),
        "fields": MODEL_Z_WWW
        + MODEL_Z_PUBMEDID
        + MODEL_Z_DOI
        + MODEL_Z_LICZBA_CYTOWAN
        + MODEL_Z_MIEJSCEM_PRZECHOWYWANIA,
    },
)


def js_openwin(url, handle, options):
    options = ",".join([f"{a}={b}" for a, b in list(options.items())])
    d = dict(url=url, handle=handle, options=options)
    return "window.open('%(url)s','\\%(handle)s','%(options)s')" % d


NIZSZE_TEXTFIELD_Z_MAPA_ZNAKOW = {
    models.TextField: {
        "widget": Textarea(attrs={"rows": 2, "cols": 90, "class": "charmap"})
    },
}


class DomyslnyStatusKorektyMixin:
    status_korekty = forms.ModelChoiceField(
        required=True,
        queryset=Status_Korekty.objects.all(),
        initial=lambda: Status_Korekty.objects.first(),
    )


class Wycinaj_W_z_InformacjiMixin:
    def clean_informacje(self):
        i = self.cleaned_data.get("informacje")
        if i:
            x = i.lower()
            n = 0
            if x.startswith("w:"):
                n = 2
            if x.startswith("w :"):
                n = 3
            if n:
                return i[n:].strip()
        return i


class LimitingFormset(BaseInlineFormSet):
    def get_queryset(self):
        if not hasattr(self, "_queryset_limited"):
            qs = super().get_queryset()
            self._queryset_limited = qs[:100]
        return self._queryset_limited


def link_do_obiektu(obj, friendly_name=None):
    opts = obj._meta
    obj_url = reverse(
        f"admin:{opts.app_label}_{opts.model_name}_change",
        args=(quote(obj.pk),),
        # current_app=self.admin_site.name,
    )
    # Add a link to the object's change form if the user can edit the obj.
    if friendly_name is None:
        friendly_name = mark_safe(obj)
    return format_html('<a href="{}">{}</a>', urlquote(obj_url), friendly_name)


def sprobuj_policzyc_sloty(request, obj):
    if obj.rok >= PBN_MIN_ROK and obj.rok <= PBN_MAX_ROK:
        try:
            ISlot(obj)
            messages.success(
                request,
                'Punkty dla dyscyplin dla "%s" będą mogły być obliczone.'
                % link_do_obiektu(obj),
            )
        except CannotAdapt as e:
            messages.error(
                request,
                'Nie można obliczyć punktów dla dyscyplin dla "%s": %s'
                % (link_do_obiektu(obj), e),
            )
    else:
        messages.warning(
            request,
            'Punkty dla dyscyplin dla "%s" nie będą liczone - rok poza zakresem (%i)'
            % (link_do_obiektu(obj), obj.rok),
        )


def sprobuj_wgrac_do_pbn(request, obj, force_upload=False, pbn_client=None):
    from bpp.models.uczelnia import Uczelnia

    if obj.charakter_formalny.rodzaj_pbn is None:
        messages.info(
            request,
            'Rekord "%s" nie będzie eksportowany do PBN zgodnie z ustawieniem dla charakteru formalnego.'
            % link_do_obiektu(obj),
        )
        return

    uczelnia = Uczelnia.objects.get_for_request(request)
    if uczelnia is None:
        messages.info(
            request,
            'Rekord "%s" nie zostanie wyeksportowany do PBN, ponieważ w systemie brakuje obiektu "Uczelnia", a'
            " co za tym idzie, jakchkolwiek ustawień" % link_do_obiektu(obj),
        )
        return

    if not uczelnia.pbn_integracja or not uczelnia.pbn_aktualizuj_na_biezaco:
        return

    if pbn_client is None:
        pbn_client = uczelnia.pbn_client(request.user.pbn_token)

    # Sprawdź, czy wydawnictwo nadrzędne ma odpowoednik PBN:
    if (
        hasattr(obj, "wydawnictwo_nadrzedne_id")
        and obj.wydawnictwo_nadrzedne_id is not None
    ):
        wn = obj.wydawnictwo_nadrzedne
        if wn.pbn_uid_id is None:
            messages.info(
                request,
                "Wygląda na to, że wydawnictwo nadrzędne tego rekordu nie posiada odpowiednika "
                "w PBN, spróbuję go pobrać.",
            )
            udalo = False
            if wn.isbn:
                ni = normalize_isbn(wn.isbn)
                if ni:
                    from pbn_api.integrator import _pobierz_prace_po_elemencie

                    res = None
                    try:
                        res = _pobierz_prace_po_elemencie(pbn_client, "isbn", ni)
                    except NeedsPBNAuthorisationException:
                        messages.warning(
                            request,
                            "Wyszukanie PBN UID wydawnictwa nadrzędnego po ISBN nieudane - "
                            "autoryzuj się najpierw w PBN. ",
                        )

                    if res:
                        messages.info(
                            request,
                            f"Udało się dopasować PBN UID wydawnictwa nadrzędnego po ISBN "
                            f"({', '.join([x.tytul_oryginalny for x in res])}). ",
                        )
                        udalo = True

            elif wn.doi:
                nd = normalize_isbn(wn.doi)
                if nd:
                    from pbn_api.integrator import _pobierz_prace_po_elemencie

                    res = None
                    try:
                        res = _pobierz_prace_po_elemencie(pbn_client, "doi", nd)
                    except NeedsPBNAuthorisationException:
                        messages.warning(
                            request,
                            "Wyszukanie PBN UID wydawnictwa nadrzędnego po DOI nieudane - "
                            "autoryzuj się najpierw w PBN. ",
                        )

                    if res:
                        messages.info(
                            request,
                            f"Udało się dopasować PBN UID wydawnictwa nadrzędnego po DOI. "
                            f"({', '.join([x.tytul_oryginalny for x in res])}). ",
                        )
                        udalo = True

            if not udalo:
                messages.warning(
                    request,
                    "Wygląda na to, że nie udało się dopasować rekordu nadrzędnego po ISBN/DOI do rekordu "
                    "po stronie PBN. Jeżeli jednak dokonano autoryzacji w PBN, to pewne rekordy z PBN "
                    "zostały teraz pobrane i możesz spróbować ustawić odpowiednik "
                    "PBN dla wydawnictwa nadrzędnego ręcznie. ",
                )

    try:
        uczelnia = Uczelnia.objects.get_for_request(request)

        pbn_client.sync_publication(
            obj,
            force_upload=force_upload,
            delete_statements_before_upload=uczelnia.pbn_api_kasuj_przed_wysylka,
            export_pk_zero=not uczelnia.pbn_api_nie_wysylaj_prac_bez_pk,
            always_affiliate_to_uid=uczelnia.pbn_uid_id
            if uczelnia.pbn_api_afiliacja_zawsze_na_uczelnie
            else None,
        )

    except SameDataUploadedRecently as e:
        link_do_wyslanych = reverse(
            "admin:pbn_api_sentdata_change",
            args=(SentData.objects.get_for_rec(obj).pk,),
        )

        messages.info(
            request,
            f'Identyczne dane rekordu "{link_do_obiektu(obj)}" zostały wgrane do PBN w dniu {e}. '
            f"Nie aktualizuję w PBN API. Jeżeli chcesz wysłać ten rekord do PBN, musisz dokonać jakiejś zmiany "
            f"danych rekodu lub "
            f'usunąć informacje o <a target=_blank href="{link_do_wyslanych}">wcześniej wysłanych danych do PBN</a> '
            f"(Redagowanie -> PBN API -> Wysłane informacje). "
            f'<a target=_blank href="{obj.link_do_pbn()}">Kliknij tutaj, aby otworzyć w PBN</a>. ',
        )
        return

    except AccessDeniedException as e:
        messages.warning(
            request,
            f'Nie można zsynchronizować obiektu "{link_do_obiektu(obj)}" z PBN pod adresem '
            f"API {e.url}. Brak dostępu -- najprawdopodobniej użytkownik nie posiada odpowiednich uprawnień "
            f"po stronie PBN/POLON. ",
        )
        return

    except PKZeroExportDisabled:
        messages.warning(
            request,
            f"Eksport prac z PK=0 jest wyłączony w konfiguracji. Próba wysyłki do PBN rekordu "
            f'"{link_do_obiektu(obj)}" nie została podjęta z uwagi na konfigurację systemu. ',
        )
        return

    except NeedsPBNAuthorisationException as e:
        messages.warning(
            request,
            f'Nie można zsynchronizować obiektu "{link_do_obiektu(obj)}" z PBN pod adresem '
            f"API {e.url}. Brak dostępu z powodu nieprawidłowego tokena -- wymagana autoryzacja w PBN. "
            f'<a target=_blank href="{reverse("pbn_api:authorize")}">Kliknij tutaj, aby autoryzować sesję</a>.',
        )
        return

    except Exception as e:
        try:
            link_do_wyslanych = reverse(
                "admin:pbn_api_sentdata_change",
                args=(SentData.objects.get_for_rec(obj).pk,),
            )
        except SentData.DoesNotExist:
            link_do_wyslanych = None

        extra = ""
        if link_do_wyslanych:
            extra = (
                '<a target=_blank href="%s">Kliknij, aby otworzyć widok wysłanych danych</a>.'
                % link_do_wyslanych
            )

        messages.warning(
            request,
            'Nie można zsynchronizować obiektu "%s" z PBN. Kod błędu: %r. %s'
            % (link_do_obiektu(obj), e, extra),
        )
        return

    sent_data = SentData.objects.get(
        content_type=ContentType.objects.get_for_model(obj), object_id=obj.pk
    )
    sent_data_link = link_do_obiektu(
        sent_data, "Kliknij tutaj, aby otworzyć widok wysłanych danych. "
    )

    publication_link = link_do_obiektu(
        sent_data.pbn_uid,
        "Kliknij tutaj, aby otworzyć zwrotnie otrzymane z PBN dane o rekordzie. ",
    )
    messages.success(
        request,
        f"Dane w PBN dla rekordu {link_do_obiektu(obj)} zostały zaktualizowane. "
        f'<a target=_blank href="{obj.link_do_pbn()}">Kliknij tutaj, aby otworzyć w PBN</a>. '
        f"{sent_data_link}{publication_link}",
    )


def get_rekord_id_from_GET_qs(request):
    flt = request.GET.get("_changelist_filters", "?")
    data = parse_qs(flt)  # noqa
    if "rekord__id__exact" in data:
        try:
            return int(data.get("rekord__id__exact")[0])
        except (ValueError, TypeError):
            pass


class OptionalPBNSaveMixin:
    def render_change_form(
        self, request, context, add=False, change=False, form_url="", obj=None
    ):
        from bpp.models import Uczelnia

        uczelnia = Uczelnia.objects.get_default()
        if uczelnia is not None:
            if uczelnia.pbn_integracja and uczelnia.pbn_aktualizuj_na_biezaco:
                context.update({"show_save_and_pbn": True})

        return super().render_change_form(request, context, add, change, form_url, obj)

    def response_post_save_change(self, request, obj):
        if "_continue_and_pbn" in request.POST:
            sprobuj_wgrac_do_pbn(request, obj)

            opts = self.model._meta
            route = f"admin:{opts.app_label}_{opts.model_name}_change"

            post_url = reverse(route, args=(obj.pk,))

            from django.http import HttpResponseRedirect

            return HttpResponseRedirect(post_url)
        else:
            # Otherwise, use default behavior
            return super().response_post_save_change(request, obj)


def sprawdz_duplikaty_www_doi(request, obj):
    from bpp.models.cache import Rekord

    IEXACT = "__iexact"
    for field, operator, label in [
        ("www", IEXACT, const.WWW_FIELD_LABEL),
        ("public_www", IEXACT, const.PUBLIC_WWW_FIELD_LABEL),
        ("doi", IEXACT, const.DOI_FIELD_LABEL),
        ("pbn_uid_id", "", const.PBN_UID_FIELD_LABEL),
    ]:
        if not hasattr(obj, field):
            continue

        v = getattr(obj, field)
        if v in [None, ""]:
            continue

        rekord_pk = (ContentType.objects.get_for_model(obj).pk, obj.pk)

        query = Q(**{field + operator: v})

        if field == "www":
            query |= Q(public_www__iexact=v)
        elif field == "public_www":
            query |= Q(www__iexact=v)

        if Rekord.objects.filter(query).exclude(pk=rekord_pk).exists():
            messages.warning(
                request, const.ZDUBLOWANE_POLE_KOMUNIKAT.format(label=label)
            )



from django.db import models
from django.db.models.deletion import DO_NOTHING
from django.db.models.query import QuerySet
from django_group_by import GroupByMixin

from django.contrib.postgres.fields import ArrayField

from bpp.fields import YearField
from bpp.models import Autor_Dyscyplina, Autorzy, Rekord


class RaportZerowyQuerySet(QuerySet, GroupByMixin):
    pass


class RaportZerowyEntry(models.Model):
    objects = RaportZerowyQuerySet.as_manager()

    id = models.IntegerField(primary_key=True)
    autor = models.ForeignKey("bpp.Autor", DO_NOTHING)  # , primary_key=True)
    rok = YearField()
    dyscyplina_naukowa = models.ForeignKey("bpp.Dyscyplina_Naukowa", DO_NOTHING)

    class Meta:
        managed = False
        ordering = ("autor", "dyscyplina_naukowa")


class RaportUczelniaEwaluacjaView(models.Model):
    id = ArrayField(models.PositiveIntegerField(), 4, primary_key=True)

    rekord = models.ForeignKey(Rekord, DO_NOTHING, related_name="+")
    autorzy = models.ForeignKey(Autorzy, DO_NOTHING, related_name="+")
    autor_dyscyplina = models.ForeignKey(Autor_Dyscyplina, DO_NOTHING, related_name="+")

    autorzy_z_dyscypliny = ArrayField(models.TextField())
    pkdaut = models.DecimalField(max_digits=20, decimal_places=4)
    slot = models.DecimalField(max_digits=20, decimal_places=4)

    class Meta:
        managed = False
        ordering = ("rekord__tytul_oryginalny", "autorzy__kolejnosc")
        db_table = "bpp_uczelnia_ewaluacja_view"


class RaportEwaluacjaUpowaznieniaView(models.Model):
    id = ArrayField(models.PositiveIntegerField(), 4, primary_key=True)

    rekord = models.ForeignKey(Rekord, DO_NOTHING, related_name="+")
    autorzy = models.ForeignKey(Autorzy, DO_NOTHING, related_name="+")
    autor_dyscyplina = models.ForeignKey(Autor_Dyscyplina, DO_NOTHING, related_name="+")

    class Meta:
        managed = False
        db_table = "bpp_ewaluacja_upowaznienia_view"



from zglos_publikacje.models import Obslugujacy_Zgloszenia_Wydzialow


def test_Obslugujacy_Zgloszenia_WydzialowManager_emaile_dla_wydzialu(
    wydzial, normal_django_user
):
    normal_django_user.email = "foo@bar.pl"
    normal_django_user.save()

    assert Obslugujacy_Zgloszenia_Wydzialow.objects.emaile_dla_wydzialu(wydzial) is None

    Obslugujacy_Zgloszenia_Wydzialow.objects.create(
        user=normal_django_user, wydzial=wydzial
    )
    assert Obslugujacy_Zgloszenia_Wydzialow.objects.emaile_dla_wydzialu(wydzial) == [
        normal_django_user.email
    ]



from builtins import callable

from adminsortable2.admin import SortableAdminMixin
from django.core.exceptions import FieldDoesNotExist

from dynamic_columns.models import ModelAdminColumn

from django.contrib import admin
from django.contrib.admin import ModelAdmin as DjangoModelAdmin
from django.contrib.admin.utils import NotRelationField, get_model_from_relation

from django.utils.translation import gettext_lazy as _


@admin.action(description=_("Enable selected columns"))
def make_enabled(modeladmin, request, queryset):
    queryset.update(enabled=True)


@admin.action(description=_("Disable selected columns"))
def make_disabled(modeladmin, request, queryset):
    queryset.update(enabled=False)


@admin.register(ModelAdminColumn)
class ModelAdminColumnAdmin(SortableAdminMixin, DjangoModelAdmin):

    ordering = ["ordering"]
    list_filter = ["parent", "enabled"]
    list_display = ["col_parent_name", "col_verbose_name", "enabled", "ordering"]
    readonly_fields = ["parent", "col_name", "ordering"]
    actions = [make_disabled, make_enabled]

    def has_delete_permission(self, request, obj=None):
        # By default ModelAdminColumns are managed automatically, so there is no
        # need to delete them
        return False

    def has_add_permission(self, request):
        # See ``self.has_delete_permission`` comment
        return False

    def col_parent_name(self, obj: ModelAdminColumn):
        """Verbose name of model in column's ModelAdmin"""
        class_ = obj.parent.model_ref.model_class()
        return class_._meta.verbose_name

    col_parent_name.short_description = _("Model admin name")

    def col_verbose_name(self, obj: ModelAdminColumn):
        """Get verbose name of ModelAdminColumn using various strategies"""
        model = obj.parent.model_ref.model_class()

        # If obj.col_name is a field, use its name later:
        try:
            field = model._meta.get_field(obj.col_name)
        except FieldDoesNotExist:
            # This is not a field. This could be Django's ModelAdmin callable
            model_admin_callable = None
            try:
                # Try getting that callable.
                model_admin_callable = getattr(obj.parent.class_ref, obj.col_name)
            except AttributeError:
                # If callable does not exists, return just the name of the column. There
                # is no such field and no such callable - maybe this is a problem with
                # non-existent column name?
                return obj.col_name

            if model_admin_callable and callable(model_admin_callable):
                # It is a function. Does it have a short_description?
                try:
                    return model_admin_callable.short_description
                except AttributeError:
                    # It is a function, but it does not have ``short_description``
                    # attribute. Let's return the col_name then:
                    return obj.col_name

        # It is a field!
        ret = obj.col_name

        if (
            hasattr(field, "verbose_name")
            and field.verbose_name
            and field.verbose_name != obj.col_name
        ):
            # Does it have a non-empty ``verbose_name`` attribute?
            ret = field.verbose_name
        else:
            # It does not have a ``verbose_name`` attribute.
            # Is it a ForeignKeyField?
            other_model = None
            try:
                other_model = get_model_from_relation(field)
            except NotRelationField:
                pass

            if other_model:
                # It is a ForeignKey! Does it have ``verbose_name``?
                try:
                    ret = other_model._meta.verbose_name
                except AttributeError:
                    pass

        return ret

    col_verbose_name.short_description = _("Column name")



"""W tym pakciecie bpp.reports znajdują się raporty, które generowane są
za pomocą celery.
"""

import re

from celeryui.registry import registerAdapter

# Bo padną importy reports/__init__.py w chwili importowania formularzy

numery_stron_regex = re.compile(
    ".*((s|str)\\.\\s*(?P<strony>((\\w+)\\-(\\w+)|(\\w+)))).*"
)


def wytnij_numery_stron(s):
    """Wycina numery stron z pola szczegóły"""
    if s is None:
        return
    m = numery_stron_regex.match(s)
    if m is not None:
        return m.group("strony")


tomy_regex = re.compile(".*((vol\\.|tom|t\\.) (?P<tom>\\d+)).*")


def wytnij_tom(s):
    """
    Wycina tom po regexpie:
     * tom X
     * t. X
     * vol. X

    :param s: string
    :type s: str
    :return:
    :rtype: str
    """
    if s is None:
        return
    m = tomy_regex.match(s)
    if m:
        return m.group("tom")


def wytnij_zbedne_informacje_ze_zrodla(z):

    if z.startswith("W: "):
        z = z[3:]

    def splituj(v, co):
        if v.find(co):
            v = v.split(co)[0]
        return v

    for elem in [
        "Pod red. ",
        " Ed. ",
        "Red. ",
        "Pod. red. ",
        " Red. nauk. ",
        "Red. nauk. ",
        "Sci. ed. ",
        " Eds. ",
        "[Ed. by]",
        "Praca zbior.",
        "[Ed.]",
        "Sci. eds.",
        "Pr. zbior.",
        "Praca zbiorowa pod",
        "Aut.",
        "[Red.]",
        "Sci.ed.",
        "Praca zbiorowa",
        ": praca zbiorowa",
        "Ed.",
        "Edited by",
        "Wyd.",
        ": praca zbiorowa",
        " : [księga dedykowana",
    ]:
        z = splituj(z, elem)

    return z.strip()


# -*- encoding: utf-8 -*-


def slugify(title):
    return title.lower().replace(" ", "-")


import logging

logger = logging.getLogger(__name__)


def addToRegistry(klass):
    logger.info(f"rejestruje {klass.slug!r} jako {klass!r}")
    registerAdapter(klass.slug, klass)


# Poniższy import jest KONIECZNY żeby adaptery do registry się
# prawidłowo zassały (tasks.py importuje tylko toplevel-module)



import django_filters
from rest_framework import viewsets

from api_v1.serializers.patent import Patent_AutorSerializer, PatentSerializer
from api_v1.viewsets.common import UkryjStatusyKorektyMixin
from bpp.models import Patent, Patent_Autor


class Patent_AutorViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Patent_Autor.objects.all()
    serializer_class = Patent_AutorSerializer


class PatentFilterSet(django_filters.rest_framework.FilterSet):
    ostatnio_zmieniony = django_filters.DateTimeFromToRangeFilter("ostatnio_zmieniony")
    rok = django_filters.RangeFilter("rok")

    class Meta:
        fields = ["ostatnio_zmieniony", "rok"]
        model = Patent


class PatentViewSet(UkryjStatusyKorektyMixin, viewsets.ReadOnlyModelViewSet):
    # Lista musi być posortowana po PK aby nie było duplikatów
    queryset = (
        Patent.objects.exclude(nie_eksportuj_przez_api=True)
        .order_by("pk")
        .select_related()  # "status_korekty", "jezyk", "charakter_formalny")
        .prefetch_related("autorzy_set", "slowa_kluczowe")
    )
    serializer_class = PatentSerializer
    filterset_class = PatentFilterSet



# -*- encoding: utf-8 -*-

from django.core.management import BaseCommand
from django.db import transaction

from bpp.models import (
    Autor_Dyscyplina,
    Wydawnictwo_Ciagle_Autor,
    Wydawnictwo_Zwarte_Autor,
)
from bpp.models import cache
import logging

logger = logging.getLogger("django")


class Command(BaseCommand):
    help = "Przypisuje dyscypliny do rekordów autorom, na podstawie wpisów Autor_Dyscyplina"

    def add_arguments(self, parser):
        parser.add_argument(
            "--ustawiaj-pierwsza-gdy-dwie",
            action="store_true",
            help="""Dla osób posiadających dwie dysycpliny, ustawiaj zawsze pierwszą dyscyplinę.
            Jeżeli ta opcja nie zostanie określona, osoby posiadające dwie dysycpliny nie będą
            miały ustawianej dyscypliny w swoich rekordach. """,
        )
        parser.add_argument("--disable-cache", action="store_true")

    @transaction.atomic
    def handle(self, verbosity, *args, **options):
        if verbosity > 1:
            logger.setLevel(logging.INFO)

        if verbosity > 2:
            logger.setLevel(logging.DEBUG)

        logger.debug("ID autora\tAutor\tRok\tDyscyplina\tTytul pracy\tID pracy")
        query = Autor_Dyscyplina.objects.all().exclude(dyscyplina_naukowa=None)

        if not options["ustawiaj_pierwsza_gdy_dwie"]:
            query = query.filter(subdyscyplina_naukowa=None)

        if options["disable_cache"]:
            cache.disable()

        for ad in query:
            for klass in Wydawnictwo_Zwarte_Autor, Wydawnictwo_Ciagle_Autor:
                for instance in klass.objects.filter(
                    autor=ad.autor, rekord__rok=ad.rok, dyscyplina_naukowa=None
                ):
                    logger.debug(
                        f"{ad.autor.pk}\t{ad.autor}\t{ad.rok}\t"
                        f"{ad.dyscyplina_naukowa}\t{instance.rekord.tytul_oryginalny}"
                        f"\t{instance.rekord.pk}"
                    )
                    instance.dyscyplina_naukowa = ad.dyscyplina_naukowa

                    if ad.subdyscyplina_naukowa_id is not None:
                        logger.info(
                            f"Autor {ad.autor} ma za rok {ad.rok} dwie dyscypliny, "
                            f"przypisuję pierwszą do pracy {instance.rekord.tytul_oryginalny} "
                            f"{instance.rekord.rok}"
                        )

                    instance.save()



# Generated by Django 3.0.14 on 2021-07-13 20:29

import django.db.models.deletion
from django.db import migrations, models

import django.contrib.postgres.fields.jsonb


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0012_auto_20210621_0947"),
    ]

    operations = [
        migrations.CreateModel(
            name="OswiadczenieInstytucji",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("addedTimestamp", models.DateField()),
                ("area", models.PositiveSmallIntegerField()),
                ("inOrcid", models.BooleanField()),
                ("type", models.CharField(max_length=50)),
                (
                    "institutionId",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="pbn_api.Institution",
                    ),
                ),
                (
                    "personId",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="pbn_api.Scientist",
                    ),
                ),
                (
                    "publicationId",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="pbn_api.Publication",
                    ),
                ),
            ],
        ),
        migrations.CreateModel(
            name="PublikacjaInstytucji",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("publicationType", models.CharField(max_length=50)),
                ("publicationVersion", models.UUIDField()),
                ("publicationYear", models.PositiveSmallIntegerField()),
                ("snapshot", django.contrib.postgres.fields.jsonb.JSONField()),
                (
                    "insPersonId",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="pbn_api.Scientist",
                    ),
                ),
                (
                    "institutionId",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="pbn_api.Institution",
                    ),
                ),
                (
                    "publicationId",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="pbn_api.Publication",
                    ),
                ),
            ],
            options={
                "unique_together": {("insPersonId", "institutionId", "publicationId")},
            },
        ),
    ]



from django.contrib.contenttypes.models import ContentType

from django_bpp.celery_tasks import app
from long_running.exceptions import (
    ObjectDoesNotExistException,
    ProcessingAlreadyStartedException,
)


@app.task
def perform_generic_long_running_task(
    app_label, model, pk, no_tries=0, max_tries=10, delay_between_tries=5
):

    if no_tries >= max_tries:
        raise ObjectDoesNotExistException(app_label, model, pk, no_tries, max_tries)

    ct = ContentType.objects.get_by_natural_key(app_label, model)
    klass = ct.model_class()

    try:
        obj = klass.objects.get(pk=pk)
    except klass.DoesNotExist:
        # retry this task with `delay_between_tries` into the future
        return perform_generic_long_running_task.apply_async(
            args=(app_label, model, pk, no_tries + 1, max_tries, delay_between_tries),
            countdown=delay_between_tries,
        )

    if obj.started_on is not None:
        raise ProcessingAlreadyStartedException(obj)

    obj.task_perform()



import pytest

from pbn_api.tests.utils import middleware

from django.contrib.messages import get_messages

from bpp.admin.helpers import sprobuj_policzyc_sloty
from bpp.const import PBN_MAX_ROK, PBN_MIN_ROK


@pytest.mark.django_db
def test_sprobuj_policzyc_sloty(rf, zwarte_z_dyscyplinami):
    req = rf.get("/")
    for a in range(PBN_MIN_ROK, PBN_MAX_ROK + 1):
        zwarte_z_dyscyplinami.rok = a
        zwarte_z_dyscyplinami.save()

        with middleware(req):
            sprobuj_policzyc_sloty(req, zwarte_z_dyscyplinami)

        msg = get_messages(req)
        assert (
            'Wydawnictwo Zwarte ĄćłłóńŹ</a>" będą mogły być obliczone.'
            in list(msg)[0].message
        )



import pytest

from long_running.notification_mixins import (
    ASGINotificationMixin,
    NullNotificationMixin,
)


@pytest.fixture
def fake_rekord():
    class FakeRekord(ASGINotificationMixin):
        pk = 500

    return FakeRekord()


def test_ASGINotificationMixin_asgi_channel_name(fake_rekord):
    assert fake_rekord.asgi_channel_name == "500"


def test_ASGINotificationMixin_send_notification(fake_rekord, mocker):
    from notifications import core

    mocker.patch("notifications.core.send_notification")
    fake_rekord.send_notification("msg")
    core.send_notification.assert_called_once_with("500", 20, "msg")


def test_ASGINotificationMixin_send_progress(fake_rekord, mocker):
    from notifications import core

    mocker.patch("notifications.core._send")
    fake_rekord.send_progress(20)
    core._send.assert_called_once()


@pytest.mark.django_db
def test_ASGINotificationMixin_send_processing_finished(fake_rekord, mocker):
    from notifications import core

    mocker.patch("notifications.core._send")
    fake_rekord.send_processing_finished()
    core._send.assert_called()


def tesT_NullNotificationMixin():
    x = NullNotificationMixin()
    x.send_notification()
    x.send_processing_finished()
    assert True



# Generated by Django 3.2.15 on 2022-10-09 15:31

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0332_pbn_zawsze_uid_uczelni"),
        ("zglos_publikacje", "0013_auto_20220910_2114"),
    ]

    operations = [
        migrations.AddField(
            model_name="zgloszenie_publikacji_autor",
            name="kierunek_studiow",
            field=models.ForeignKey(
                blank=True,
                null=True,
                on_delete=django.db.models.deletion.SET_NULL,
                to="bpp.kierunek_studiow",
            ),
        ),
    ]



# Generated by Django 2.1.7 on 2019-08-23 21:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0173_auto_20190812_1412'),
    ]

    operations = [
        migrations.AddField(
            model_name='uczelnia',
            name='podpowiadaj_dyscypliny',
            field=models.BooleanField(default=True, help_text='W sytuacji gdy to pole ma wartość "PRAWDA", system będzie podpowiadał dyscyplinę\n        naukową dla powiązania rekordu wydawnictwa i autora w sytuacji, gdy autor ma na dany rok\n        określoną tylko jedną dyscyplinę. W sytuacji przypisania dla autora dwóch dyscyplin na dany rok, \n        pożądaną dyscyplinę będzie trzeba wybrać ręcznie, niezależnie od ustawienia tego pola. '),
        ),
    ]



# Generated by Django 3.0.11 on 2021-03-08 11:46

from decimal import Decimal

import django.contrib.postgres.fields.jsonb
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0245_auto_20210308_1246"),
        ("import_list_if", "0001_initial"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="importlistif",
            options={},
        ),
        migrations.CreateModel(
            name="ImportListIfRow",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("dane_z_xls", django.contrib.postgres.fields.jsonb.JSONField()),
                (
                    "impact_factor",
                    models.DecimalField(
                        blank=True,
                        decimal_places=3,
                        default=Decimal("0.000"),
                        max_digits=6,
                        null=True,
                    ),
                ),
                ("zintegrowano", models.BooleanField(default=False)),
                (
                    "parent",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        to="import_list_if.ImportListIf",
                    ),
                ),
                (
                    "zrodlo",
                    models.ForeignKey(
                        blank=True,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to="bpp.Zrodlo",
                    ),
                ),
            ],
        ),
    ]



import pytest
from model_bakery import baker

from fixtures.pbn_api import MOCK_RETURNED_MONGODB_DATA
from pbn_api.adapters.wydawnictwo import WydawnictwoPBNAdapter
from pbn_api.client import (
    PBN_GET_INSTITUTION_STATEMENTS,
    PBN_GET_PUBLICATION_BY_ID_URL,
    PBN_POST_PUBLICATIONS_URL,
)
from pbn_api.exceptions import AccessDeniedException
from pbn_api.models import SentData
from pbn_api.tests.utils import middleware

from django.contrib.messages import get_messages

from bpp.admin.helpers import sprobuj_wgrac_do_pbn
from bpp.models import Charakter_Formalny


@pytest.mark.django_db
def test_sprobuj_wgrac_do_pbn_charakter_zly(
    rf,
    pbn_wydawnictwo_zwarte,
):
    req = rf.get("/")

    pbn_wydawnictwo_zwarte.charakter_formalny = baker.make(
        Charakter_Formalny, rodzaj_pbn=None
    )

    with middleware(req):
        sprobuj_wgrac_do_pbn(req, pbn_wydawnictwo_zwarte)

    msg = get_messages(req)

    assert "nie będzie eksportowany do PBN" in list(msg)[0].message


@pytest.mark.django_db
def test_sprobuj_wgrac_do_pbn_uczelnia_brak_obiektu(
    rf, pbn_wydawnictwo_zwarte_z_charakterem
):
    req = rf.get("/")

    with middleware(req):
        sprobuj_wgrac_do_pbn(req, pbn_wydawnictwo_zwarte_z_charakterem)

    msg = get_messages(req)

    assert 'w systemie brakuje obiektu "Uczelnia"' in list(msg)[0].message


@pytest.mark.django_db
def test_sprobuj_wgrac_do_pbn_uczelnia_integracja_wylaczona(
    rf, pbn_wydawnictwo_zwarte_z_charakterem, uczelnia
):
    req = rf.get("/")

    uczelnia.pbn_integracja = False
    uczelnia.pbn_aktualizuj_na_biezaco = False
    uczelnia.save()

    with middleware(req):
        sprobuj_wgrac_do_pbn(req, pbn_wydawnictwo_zwarte_z_charakterem)

    msg = get_messages(req)
    assert len(msg) == 0


@pytest.mark.django_db
def test_sprobuj_wgrac_do_pbn_dane_juz_wyslane(
    pbn_wydawnictwo_zwarte_z_charakterem, pbn_uczelnia, pbn_client, rf
):
    SentData.objects.updated(
        pbn_wydawnictwo_zwarte_z_charakterem,
        WydawnictwoPBNAdapter(pbn_wydawnictwo_zwarte_z_charakterem).pbn_get_json(),
    )

    req = rf.get("/")

    with middleware(req):
        sprobuj_wgrac_do_pbn(
            req, pbn_wydawnictwo_zwarte_z_charakterem, pbn_client=pbn_client
        )

    msg = get_messages(req)
    assert "Identyczne dane rekordu" in list(msg)[0].message


@pytest.mark.django_db
def test_sprobuj_wgrac_do_pbn_access_denied(
    pbn_wydawnictwo_zwarte_z_charakterem, pbn_client, rf, pbn_uczelnia
):
    req = rf.get("/")

    pbn_client.transport.return_values[
        PBN_POST_PUBLICATIONS_URL
    ] = AccessDeniedException(url="foo", content="testujemy")

    with middleware(req):
        sprobuj_wgrac_do_pbn(
            req, pbn_wydawnictwo_zwarte_z_charakterem, pbn_client=pbn_client
        )

    msg = get_messages(req)
    assert "Brak dostępu --" in list(msg)[0].message


@pytest.mark.django_db
def test_sprobuj_wgrac_do_pbn_inny_exception(
    pbn_wydawnictwo_zwarte_z_charakterem, pbn_client, rf, pbn_uczelnia
):
    req = rf.get("/")

    pbn_client.transport.return_values[PBN_POST_PUBLICATIONS_URL] = ZeroDivisionError

    with middleware(req):
        sprobuj_wgrac_do_pbn(
            req, pbn_wydawnictwo_zwarte_z_charakterem, pbn_client=pbn_client
        )

    msg = get_messages(req)
    assert "Nie można zsynchronizować" in list(msg)[0].message


@pytest.mark.django_db
def test_sprobuj_wgrac_do_pbn_inny_blad(
    pbn_wydawnictwo_zwarte_z_charakterem, pbn_client, rf, pbn_uczelnia
):
    req = rf.get("/")

    pbn_client.transport.return_values[PBN_POST_PUBLICATIONS_URL] = Exception("test")

    with middleware(req):
        sprobuj_wgrac_do_pbn(
            req, pbn_wydawnictwo_zwarte_z_charakterem, pbn_client=pbn_client
        )

    msg = get_messages(req)
    assert "Nie można zsynchronizować" in list(msg)[0].message


@pytest.mark.django_db
def test_sprobuj_wgrac_do_pbn_sukces(
    pbn_wydawnictwo_zwarte_z_charakterem, pbn_client, rf, pbn_uczelnia
):
    req = rf.get("/")

    pbn_client.transport.return_values[PBN_POST_PUBLICATIONS_URL] = {"objectId": "123"}
    pbn_client.transport.return_values[
        PBN_GET_PUBLICATION_BY_ID_URL.format(id="123")
    ] = MOCK_RETURNED_MONGODB_DATA
    pbn_client.transport.return_values[
        PBN_GET_INSTITUTION_STATEMENTS + "?publicationId=123&size=5120"
    ] = []

    with middleware(req):
        sprobuj_wgrac_do_pbn(
            req, pbn_wydawnictwo_zwarte_z_charakterem, pbn_client=pbn_client
        )

    msg = get_messages(req)
    assert "zostały zaktualizowane" in list(msg)[0].message



# Generated by Django 2.1.7 on 2019-07-01 11:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0163_cachequeue'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='cachequeue',
            options={'ordering': ('-created_on',)},
        ),
        migrations.RemoveField(
            model_name='cachequeue',
            name='task_id',
        ),
        migrations.AlterField(
            model_name='cachequeue',
            name='created_on',
            field=models.DateTimeField(blank=True),
        ),
    ]



import sys

from django.core.management import BaseCommand
from django.db import transaction

from bpp.models import Autorzy, Jednostka, Kierunek_Studiow, Wydzial


class Command(BaseCommand):
    help = 'Mapuje kierunki studiów z zadanego "wydziału" i wyświetla zmienione rekordy'

    @transaction.atomic
    def handle(self, *args, **options):
        for wydzialokierunek in Wydzial.objects.filter(nazwa__startswith="Studenci - "):
            wnazwa = wydzialokierunek.nazwa.replace("Studenci - ", "").strip()
            try:
                wydzial = Wydzial.objects.get(nazwa=wnazwa)
            except Wydzial.DoesNotExist:
                print(f"Brak wydziału {wnazwa} -- {wydzialokierunek}")
                sys.exit(1)

            for jednostka in Jednostka.objects.filter(wydzial__nazwa=wydzialokierunek):
                nazwa = jednostka.nazwa.lower()
                skrot = jednostka.skrot.lower()

                kierunek_studiow = Kierunek_Studiow.objects.get_or_create(
                    nazwa=nazwa, skrot=skrot, wydzial=wydzial
                )[0]

                for wa in Autorzy.objects.filter(jednostka=jednostka):
                    wa.jednostka_id = -1
                    wa.kierunek_studiow = kierunek_studiow
                    print(
                        f"{wa.rekord.tytul_oryginalny}\t{wa.autor}\t{kierunek_studiow.nazwa}"
                    )
                    wa.save()

            jednostka.delete()
        wydzialokierunek.delete()



import pytest
from django.urls import reverse

from raport_slotow.models import RaportUczelniaEwaluacjaView


def test_raport_slotow_ewaluacja_parametry_view(admin_client):
    res = admin_client.get(reverse("raport_slotow:index-ewaluacja"))
    assert res.status_code == 200


def test_raport_slotow_ewaluacja_parametry_view_post(admin_client):
    res = admin_client.post(
        reverse("raport_slotow:index-ewaluacja"),
        {"od_roku": 2020, "do_roku": 2020, "_export": "html"},
    )
    assert res.status_code == 302


def test_raport_slotow_ewaluacja_raport(
    admin_client, praca_z_dyscyplina, django_assert_max_num_queries
):
    with django_assert_max_num_queries(42):
        res = admin_client.get(
            reverse("raport_slotow:raport-ewaluacja")
            + "?_export=html&od_roku=2020&do_roku=2020"
        )
    assert res.status_code == 200


def test_raport_slotow_ewaluacja_raport_xlsx(admin_client, praca_z_dyscyplina):
    res = admin_client.get(
        reverse("raport_slotow:raport-ewaluacja")
        + "?_export=xlsx&od_roku=2020&do_roku=2020"
    )
    assert res.status_code == 200


@pytest.mark.django_db
def test_RaportUczelniaEwaluacjaView_model(praca_z_dyscyplina):
    assert RaportUczelniaEwaluacjaView.objects.all().count() == 1



"""Ustawienia systemowe

groups - lista grup wraz z uprawnieniami do edycji poszczególnych obiektów.
"""
from dbtemplates.models import Template
from django.db import transaction
from favicon.models import Favicon, FaviconImg
from flexible_reports import models as flexible_models
from multiseek.models import SearchForm
from robots.models import Rule, Url

from dynamic_columns.models import ModelAdmin, ModelAdminColumn
from ewaluacja2021.models import IloscUdzialowDlaAutora, LiczbaNDlaUczelni
from formdefaults.models import FormFieldRepresentation, FormRepresentation
from import_dbf.models import B_A, B_U, Aut, Bib, Ixn, Jed, Poz, Ses, Usi, Wx2
from miniblog.models import Article
from pbn_api.models import (
    Conference,
    Institution,
    Journal,
    Publisher,
    Scientist,
    SentData,
)
from rozbieznosci_dyscyplin.models import RozbieznosciView, RozbieznosciZrodelView
from zglos_publikacje.models import (
    Obslugujacy_Zgloszenia_Wydzialow,
    Zgloszenie_Publikacji,
    Zgloszenie_Publikacji_Autor,
)

from django.contrib.auth import get_user_model
from django.contrib.auth.models import Group, Permission
from django.contrib.contenttypes.models import ContentType
from django.contrib.sites.models import Site

from bpp.const import (
    GR_RAPORTY_WYSWIETLANIE,
    GR_WPROWADZANIE_DANYCH,
    GR_ZGLOSZENIA_PUBLIKACJI,
)
from bpp.models import (
    Autor,
    Autor_Dyscyplina,
    Autor_Jednostka,
    BppMultiseekVisibility,
    Charakter_Formalny,
    Dyscyplina_Naukowa,
    Dyscyplina_Zrodla,
    Element_Repozytorium,
    Funkcja_Autora,
    Grant,
    Grant_Rekordu,
    Grupa_Pracownicza,
    Jednostka,
    Jezyk,
    Kierunek_Studiow,
    Patent,
    Patent_Autor,
    Praca_Doktorska,
    Praca_Habilitacyjna,
    Punktacja_Zrodla,
    Redakcja_Zrodla,
    Rodzaj_Prawa_Patentowego,
    Rodzaj_Zrodla,
    Status_Korekty,
    Typ_KBN,
    Typ_Odpowiedzialnosci,
    Tytul,
    Uczelnia,
    Ukryj_Status_Korekty,
    Wydawnictwo_Ciagle,
    Wydawnictwo_Ciagle_Autor,
    Wydawnictwo_Ciagle_Streszczenie,
    Wydawnictwo_Ciagle_Zewnetrzna_Baza_Danych,
    Wydawnictwo_Zwarte,
    Wydawnictwo_Zwarte_Autor,
    Wydawnictwo_Zwarte_Streszczenie,
    Wydawnictwo_Zwarte_Zewnetrzna_Baza_Danych,
    Wydzial,
    Wymiar_Etatu,
    Zewnetrzna_Baza_Danych,
    Zrodlo,
    Zrodlo_Informacji,
)
from bpp.models.konferencja import Konferencja
from bpp.models.nagroda import Nagroda, OrganPrzyznajacyNagrody
from bpp.models.openaccess import (
    Czas_Udostepnienia_OpenAccess,
    Licencja_OpenAccess,
    Tryb_OpenAccess_Wydawnictwo_Ciagle,
    Tryb_OpenAccess_Wydawnictwo_Zwarte,
    Wersja_Tekstu_OpenAccess,
)
from bpp.models.praca_habilitacyjna import Publikacja_Habilitacyjna
from bpp.models.profile import BppUser
from bpp.models.seria_wydawnicza import Seria_Wydawnicza
from bpp.models.struktura import Jednostka_Wydzial
from bpp.models.system import Charakter_PBN
from bpp.models.wydawca import Poziom_Wydawcy, Wydawca

User = get_user_model()

groups = {
    "import DBF": [Bib, B_A, Aut, Jed, Poz, B_U, Usi, Ses, Wx2, Ixn],
    "dane systemowe": [
        Charakter_Formalny,
        Kierunek_Studiow,
        Charakter_PBN,
        Funkcja_Autora,
        Zrodlo_Informacji,
        Jezyk,
        Typ_Odpowiedzialnosci,
        Rodzaj_Zrodla,
        Status_Korekty,
        Tytul,
        Typ_KBN,
        Tryb_OpenAccess_Wydawnictwo_Ciagle,
        Tryb_OpenAccess_Wydawnictwo_Zwarte,
        Czas_Udostepnienia_OpenAccess,
        Licencja_OpenAccess,
        Wersja_Tekstu_OpenAccess,
        OrganPrzyznajacyNagrody,
        Rodzaj_Prawa_Patentowego,
        Dyscyplina_Naukowa,
        Zewnetrzna_Baza_Danych,
        Grant,
        FormFieldRepresentation,
        FormRepresentation,
        Grupa_Pracownicza,
        Wymiar_Etatu,
        Journal,
        Institution,
        Conference,
        Publisher,
        Scientist,
        SentData,
        BppMultiseekVisibility,
    ],
    "struktura": [
        Uczelnia,
        LiczbaNDlaUczelni,
        Wydzial,
        Jednostka,
        Jednostka_Wydzial,
        Ukryj_Status_Korekty,
    ],
    GR_WPROWADZANIE_DANYCH: [
        Zrodlo,
        Autor,
        Autor_Dyscyplina,
        Wydawnictwo_Ciagle,
        Wydawnictwo_Zwarte,
        Punktacja_Zrodla,
        Dyscyplina_Zrodla,
        Wydawnictwo_Ciagle_Autor,
        Wydawnictwo_Zwarte_Autor,
        Autor_Jednostka,
        Redakcja_Zrodla,
        Praca_Doktorska,
        Praca_Habilitacyjna,
        Patent,
        Patent_Autor,
        Publikacja_Habilitacyjna,
        Konferencja,
        Seria_Wydawnicza,
        Nagroda,
        Wydawnictwo_Ciagle_Zewnetrzna_Baza_Danych,
        Wydawnictwo_Ciagle_Streszczenie,
        Wydawnictwo_Zwarte_Zewnetrzna_Baza_Danych,
        Wydawnictwo_Zwarte_Streszczenie,
        Wydawca,
        Poziom_Wydawcy,
        Grant_Rekordu,
        Element_Repozytorium,
        IloscUdzialowDlaAutora,
        RozbieznosciView,
        RozbieznosciZrodelView,
    ],
    "indeks autorów": [Autor, Autor_Jednostka],
    "administracja": [
        User,
        Group,
        SearchForm,
        Obslugujacy_Zgloszenia_Wydzialow,
        ModelAdminColumn,
        ModelAdmin,
    ],
    "web": [Url, Rule, Site, Favicon, FaviconImg, Article, Template],
    "raporty": [
        flexible_models.Report,
        flexible_models.ReportElement,
        flexible_models.Table,
        flexible_models.Column,
        flexible_models.Datasource,
        flexible_models.ColumnOrder,
    ],
    GR_ZGLOSZENIA_PUBLIKACJI: [Zgloszenie_Publikacji, Zgloszenie_Publikacji_Autor],
    GR_RAPORTY_WYSWIETLANIE: [],
}

# Słownik na podstawie którego dodajemy użytkowników z danej grupy automatycznie
# do innej:
groups_auto_add = {
    GR_WPROWADZANIE_DANYCH: [
        GR_RAPORTY_WYSWIETLANIE,
    ]
}

# Po migracji, upewnij się że robots.txt są generowane poprawnie

DISALLOW_URLS = [
    "/multiseek/",
    "/bpp/raporty/",
    "/eksport_pbn/",
    "/admin/",
    "/integrator2/",
    "/password_change/",
]


def ustaw_robots_txt(**kwargs):
    urls = set()
    for elem in DISALLOW_URLS:
        url, _ignore = Url.objects.get_or_create(pattern=elem)
        urls.add(url)

    cnt = Site.objects.all().count()
    if cnt != 1:
        raise Exception("Not supported count=%i" % cnt)

    r, _ignore = Rule.objects.get_or_create(robot="*")
    r.sites.add(Site.objects.all()[0])
    for elem in DISALLOW_URLS:
        r.disallowed.add(Url.objects.get(pattern=elem))
    r.save()


@transaction.atomic
def odtworz_grupy(**kwargs):
    grp_dict = {}
    for u in BppUser.objects.all():
        grp_dict[u] = [grp.name for grp in u.groups.all()]

    for name, models in list(groups.items()):
        try:
            Group.objects.get(name=name).delete()
        except Group.DoesNotExist:
            pass

        g = Group.objects.create(name=name)
        for model in models:
            content_type = ContentType.objects.get_for_model(model)
            for permission in Permission.objects.filter(content_type=content_type):
                g.permissions.add(permission)

    for u, grps in list(grp_dict.items()):
        for gname in grps:
            u.groups.add(Group.objects.get(name=gname))
            if gname in groups_auto_add:
                for extra_group in groups_auto_add[gname]:
                    u.groups.add(Group.objects.get(name=extra_group))



# Generated by Django 3.0.11 on 2021-03-12 11:28

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0245_auto_20210308_1246"),
    ]

    operations = [
        migrations.AlterModelOptions(
            name="autor_jednostka",
            options={
                "ordering": ["autor__nazwisko", "rozpoczal_prace", "jednostka__nazwa"],
                "verbose_name": "powiązanie autor-jednostka",
                "verbose_name_plural": "powiązania autor-jednostka",
            },
        ),
    ]



from rest_framework import serializers

from api_v1.serializers.util import AbsoluteUrlSerializerMixin

from bpp.models import Autor, Autor_Jednostka, Funkcja_Autora, Tytul


class TytulSerializer(serializers.ModelSerializer):
    class Meta:
        model = Tytul
        fields = ["id", "nazwa", "skrot"]


class Funkcja_AutoraSerializer(serializers.ModelSerializer):
    class Meta:
        model = Funkcja_Autora
        fields = ["id", "nazwa", "skrot"]


class Autor_JednostkaSerializer(serializers.HyperlinkedModelSerializer):
    autor = serializers.HyperlinkedRelatedField(
        view_name="api_v1:autor-detail", read_only=True
    )
    jednostka = serializers.HyperlinkedRelatedField(
        view_name="api_v1:jednostka-detail", read_only=True
    )
    funkcja = serializers.HyperlinkedRelatedField(
        view_name="api_v1:funkcja_autora-detail", read_only=True
    )

    class Meta:
        model = Autor_Jednostka
        fields = [
            "id",
            "autor",
            "jednostka",
            "rozpoczal_prace",
            "zakonczyl_prace",
            "funkcja",
        ]


class AutorSerializer(
    AbsoluteUrlSerializerMixin, serializers.HyperlinkedModelSerializer
):
    absolute_url = serializers.SerializerMethodField("get_absolute_url")

    aktualna_jednostka = serializers.HyperlinkedRelatedField(
        view_name="api_v1:jednostka-detail", read_only=True
    )

    tytul = serializers.HyperlinkedRelatedField(
        view_name="api_v1:tytul-detail", read_only=True
    )

    aktualna_funkcja = serializers.HyperlinkedRelatedField(
        view_name="api_v1:funkcja_autora-detail", read_only=True
    )

    jednostki = serializers.HyperlinkedRelatedField(
        many=True, read_only=True, view_name="api_v1:autor_jednostka-detail"
    )

    class Meta:
        model = Autor
        fields = [
            "id",
            "imiona",
            "nazwisko",
            "tytul",
            "aktualna_jednostka",
            "aktualna_funkcja",
            "pokazuj",
            "email",
            "www",
            "urodzony",
            "zmarl",
            "poprzednie_nazwiska",
            "orcid",
            "pbn_id",
            "expertus_id",
            "slug",
            "jednostki",
            "absolute_url",
            #
            "ostatnio_zmieniony",
        ]



# Generated by Django 3.2.14 on 2022-07-07 22:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0034_auto_20211028_0341"),
    ]

    operations = [
        migrations.AlterField(
            model_name="conference",
            name="versions",
            field=models.JSONField(),
        ),
        migrations.AlterField(
            model_name="institution",
            name="versions",
            field=models.JSONField(),
        ),
        migrations.AlterField(
            model_name="journal",
            name="versions",
            field=models.JSONField(),
        ),
        migrations.AlterField(
            model_name="language",
            name="language",
            field=models.JSONField(),
        ),
        migrations.AlterField(
            model_name="publication",
            name="versions",
            field=models.JSONField(),
        ),
        migrations.AlterField(
            model_name="publikacjainstytucji",
            name="snapshot",
            field=models.JSONField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name="publisher",
            name="versions",
            field=models.JSONField(),
        ),
        migrations.AlterField(
            model_name="scientist",
            name="from_institution_api",
            field=models.BooleanField(
                db_index=True, null=True, verbose_name="Rekord z API instytucji"
            ),
        ),
        migrations.AlterField(
            model_name="scientist",
            name="versions",
            field=models.JSONField(),
        ),
        migrations.AlterField(
            model_name="sentdata",
            name="data_sent",
            field=models.JSONField(verbose_name="Wysłane dane"),
        ),
    ]



try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

import pytest

from bpp.tests.util import (
    CURRENT_YEAR,
    any_autor,
    any_ciagle,
    any_jednostka,
    select_select2_autocomplete,
)

from django_bpp.selenium_util import wait_for, wait_for_page_load

# @pytest.fixture
# def preauth_browser(preauth_browser, asgi_live_server):
#     with wait_for_page_load(preauth_browser):
#         preauth_browser.visit(asgi_live_server.url + reverse("bpp:raporty"))
#     return preauth_browser


def wybrany(browser):
    return browser.execute_script(
        "$('section.active div[data-slug]').attr('data-slug')"
    )


def submit_page(browser):
    browser.execute_script("$('input[name=submit]:visible').click()")


pytestmark = [pytest.mark.slow, pytest.mark.selenium]


@pytest.mark.django_db
@pytest.fixture
def jednostka_raportow(
    typy_odpowiedzialnosci, jezyki, statusy_korekt, typy_kbn, charaktery_formalne
):
    j = any_jednostka(nazwa="Jednostka")
    a = any_autor()

    c = any_ciagle(rok=CURRENT_YEAR)
    c.dodaj_autora(a, j)

    d = any_ciagle(rok=CURRENT_YEAR - 1)
    d.dodaj_autora(a, j)

    e = any_ciagle(rok=CURRENT_YEAR - 2)
    e.dodaj_autora(a, j)

    return j


@pytest.mark.django_db(transaction=True)
def test_ranking_autorow(preauth_browser, jednostka_raportow, asgi_live_server):
    preauth_browser.visit(
        asgi_live_server.url + reverse("bpp:ranking_autorow_formularz")
    )
    from django.utils import timezone

    now = timezone.now()

    if now.month < 2:
        val = CURRENT_YEAR - 1
    else:
        val = CURRENT_YEAR

    assert 'value="%s"' % val in preauth_browser.html


@pytest.mark.django_db(transaction=True)
def test_raport_jednostek(
    preauth_browser, jednostka_raportow, asgi_live_server, denorms
):

    with wait_for_page_load(preauth_browser):
        preauth_browser.visit(
            asgi_live_server.url + reverse("bpp:raport_jednostek_formularz")
        )

    select_select2_autocomplete(preauth_browser, "id_jednostka", "Jedn")

    preauth_browser.execute_script(
        '$("input[name=od_roku]:visible").val("' + str(CURRENT_YEAR) + '")'
    )
    preauth_browser.execute_script(
        '$("input[name=do_roku]:visible").val("' + str(CURRENT_YEAR) + '")'
    )
    with wait_for_page_load(preauth_browser):
        submit_page(preauth_browser)

    wait_for(
        lambda: f"/bpp/raporty/raport-jednostek-2012/{jednostka_raportow.pk}/{CURRENT_YEAR}/"
        in preauth_browser.url
    )



# Generated by Django 3.0.14 on 2021-09-25 19:18

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
        ("dbtemplates", "0001_initial"),
        ("bpp", "0293_pbn_api_kasowanie_przed_nie_eksp_zero"),
    ]

    operations = [
        migrations.CreateModel(
            name="SzablonDlaOpisuBibliograficznego",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "model",
                    models.OneToOneField(
                        blank=True,
                        limit_choices_to=models.Q(
                            ("app_label", "bpp"),
                            (
                                "model__in",
                                [
                                    "wydawnictwo_ciagle",
                                    "wydawnictwo_zwarte",
                                    "praca_doktorska",
                                    "praca_habilitacyjna",
                                    "patent",
                                ],
                            ),
                        ),
                        null=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        to="contenttypes.ContentType",
                    ),
                ),
                (
                    "template",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.PROTECT,
                        to="dbtemplates.Template",
                    ),
                ),
            ],
        ),
    ]



# Generated by Django 3.2.15 on 2022-10-09 15:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0331_kierunek_studiow"),
    ]

    operations = [
        migrations.AddField(
            model_name="uczelnia",
            name="pbn_api_afiliacja_zawsze_na_uczelnie",
            field=models.BooleanField(
                default=False,
                help_text="Jeżeli praca jest w jednostce z wypełnionym PBN UID bądź w jednostce innej-niż-obca, zatrudniającej-pracowników, to zaznaczenie tej opcji spowoduje, ze  zamiast PBN UID tej jednostki zostanie użyty PBN UID uczelni, co efektywnie spowoduje, że afiliacje w PBN będą na uczelnię, nie zaś na konkretną jednostkę. ",
                verbose_name="Wysyłaj zawsze PBN UID uczelni jako afiliację",
            ),
        ),
    ]



from django import template

register = template.Library()


from django.template.base import FilterExpression, kwarg_re


def parse_tag(token, parser):
    """
    Generic template tag parser.

    Returns a three-tuple: (tag_name, args, kwargs)

    tag_name is a string, the name of the tag.

    args is a list of FilterExpressions, from all the arguments that didn't look like kwargs,
    in the order they occurred, including any that were mingled amongst kwargs.

    kwargs is a dictionary mapping kwarg names to FilterExpressions, for all the arguments that
    looked like kwargs, including any that were mingled amongst args.

    (At rendering time, a FilterExpression f can be evaluated by calling f.resolve(context).)
    """
    # Split the tag content into words, respecting quoted strings.
    bits = token.split_contents()

    # Pull out the tag name.
    tag_name = bits.pop(0)

    # Parse the rest of the args, and build FilterExpressions from them so that
    # we can evaluate them later.
    args = []
    kwargs = {}
    for bit in bits:
        # Is this a kwarg or an arg?
        match = kwarg_re.match(bit)
        kwarg_format = match and match.group(1)
        if kwarg_format:
            key, value = match.groups()
            kwargs[key] = FilterExpression(value, parser)
        else:
            args.append(FilterExpression(bit, parser))

    return (tag_name, args, kwargs)


class CzyPokazywacNode(template.Node):
    def __init__(self, nodelist, attr, ignoruj_grupe=None):
        self.nodelist = nodelist

        self.attr = attr
        self.ignoruj_grupe = ignoruj_grupe

    def render(self, context):
        uczelnia = context.get("uczelnia")
        if uczelnia is not None:
            if uczelnia.sprawdz_uprawnienie(
                self.attr.token, context.request, ignoruj_grupe=self.ignoruj_grupe
            ):
                return self.nodelist.render(context)
        return ""


def czy_pokazywac(parser, token):
    tag_name, args, kwargs = parse_tag(token, parser)
    nodelist = parser.parse("end_czy_pokazywac")
    parser.delete_first_token()
    return CzyPokazywacNode(nodelist, args[0], *args[1:])


register.tag("czy_pokazywac", czy_pokazywac)



# Generated by Django 3.0.14 on 2021-05-03 22:14

from django.db import migrations

from bpp.migration_util import load_custom_sql


class Migration(migrations.Migration):

    dependencies = [
        ("rozbieznosci_dyscyplin", "0007_recreate"),
        ("bpp", "0265_rekord_mat_doi"),
    ]

    operations = [
        migrations.RunPython(
            lambda *args, **kw: load_custom_sql(
                "0002_rok_2017_i_wyzej", app_name="rozbieznosci_dyscyplin"
            )
        )
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-17 10:59
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0129_zewnetrznebazydanychview'),
    ]

    operations = [
        migrations.AddField(
            model_name='uczelnia',
            name='clarivate_password',
            field=models.CharField(blank=True, max_length=50, null=True, verbose_name='Hasło'),
        ),
        migrations.AddField(
            model_name='uczelnia',
            name='clarivate_username',
            field=models.CharField(blank=True, max_length=50, null=True, verbose_name='Nazwa użytkownika'),
        ),
        migrations.AddField(
            model_name='wydawnictwo_ciagle',
            name='liczba_cytowan',
            field=models.PositiveIntegerField(blank=True, null=True, verbose_name='Liczba cytowań'),
        ),
        migrations.AddField(
            model_name='wydawnictwo_zwarte',
            name='liczba_cytowan',
            field=models.PositiveIntegerField(blank=True, null=True, verbose_name='Liczba cytowań'),
        ),
    ]



from denorm import denorms
from django.core.management import BaseCommand
from django.db import transaction

from bpp.const import PBN_MIN_ROK
from bpp.models import Praca_Doktorska, Praca_Habilitacyjna, Wydawca, Wydawnictwo_Zwarte


class Command(BaseCommand):
    help = "Znajduje wydawców indeksowanych"

    @transaction.atomic
    def handle(self, *args, **options):
        for wydawca in Wydawca.objects.all():
            for klass in Wydawnictwo_Zwarte, Praca_Doktorska, Praca_Habilitacyjna:

                for model in klass.objects.filter(
                    wydawca=None, rok__gte=PBN_MIN_ROK
                ).filter(wydawca_opis__istartswith=wydawca.nazwa):

                    stare_wydawnictwo = model.wydawnictwo
                    model.wydawca = wydawca
                    model.wydawca_opis = model.wydawca_opis[
                        len(wydawca.nazwa) :
                    ].strip()
                    if stare_wydawnictwo.strip().lower() != model.wydawnictwo.lower():
                        print(
                            f"Nie zgadza mi sie: rekord {model} stare wydawnictwo z nowym: "
                            f"{stare_wydawnictwo} != {model.wydawnictwo}, nie zapisuję"
                        )
                    else:
                        if stare_wydawnictwo != wydawca.nazwa:
                            print(
                                "Ciąg znaków %r przypisuję do indeksowanego wydawcy ID %i nazwa %r (%r ID %s)"
                                % (
                                    stare_wydawnictwo,
                                    wydawca.pk,
                                    wydawca.nazwa,
                                    model.pk,
                                    model.tytul_oryginalny,
                                )
                            )
                        model.save()

        denorms.flush()



from .base import *  # noqa

DEBUG = False
DEBUG_TOOLBAR = False
SENDFILE_BACKEND = "sendfile.backends.nginx"

SESSION_COOKIE_SECURE = False
CSRF_COOKIE_SECURE = False

COMPRESS_ENABLED = not DEBUG
COMPRESS_OFFLINE = False

HTML_MINIFY = True

SESSION_ENGINE = "django.contrib.sessions.backends.cache"

CACHES = {
    "default": {
        "BACKEND": "redis_cache.RedisCache",
        "LOCATION": [
            f"{REDIS_HOST}:{REDIS_PORT}",  # noqa
        ],
        "OPTIONS": {
            "DB": REDIS_DB_CACHE,  # noqa
            "PARSER_CLASS": "redis.connection.HiredisParser",
            "CONNECTION_POOL_CLASS": "redis.BlockingConnectionPool",
            "CONNECTION_POOL_CLASS_KWARGS": {
                "max_connections": 50,
                "timeout": 20,
            },
            "MAX_CONNECTIONS": 1000,
            "PICKLE_VERSION": -1,
        },
    },
}

CACHE_MIDDLEWARE_SECONDS = 3600 * 24

DATABASES["default"]["CONN_MAX_AGE"] = None  # noqa

DATA_UPLOAD_MAX_MEMORY_SIZE = 2621440 * 3  # 7.5 MB
DATA_UPLOAD_MAX_NUMBER_FIELDS = 50000

CACHEOPS = {
    "bpp.bppmultiseekvisibility": {"ops": ("get", "fetch")},
    "dbtemplates.template": {"ops": ("fetch", "get")},
    "bpp.szablondlaopisubibliograficznego": {"ops": ("fetch", "get")},
    "miniblog.article": {"ops": ("get", "fetch")},
    "contenttypes.contenttype": {"ops": ("get", "fetch")},
}
CACHEOPS_REDIS = BROKER_URL  # noqa
CACHEOPS_DEFAULTS = {"timeout": 60 * 60}

ALLOWED_HOSTS = [
    "127.0.0.1",
    django_getenv("DJANGO_BPP_HOSTNAME", "localhost"),  # noqa
]

SECURE_PROXY_SSL_HEADER = ("HTTP_X_FORWARDED_PROTO", "https")

EMAIL_BACKEND = "djcelery_email.backends.CeleryEmailBackend"

# django-easy-audit

INSTALLED_APPS.append("easyaudit")  # noqa
MIDDLEWARE.append(  # noqa
    "easyaudit.middleware.easyaudit.EasyAuditMiddleware",
)

# djcelery_email
INSTALLED_APPS.append("djcelery_email")  # noqa



import contextlib

from pbn_api.client import RequestsTransport

from django.contrib.messages.middleware import MessageMiddleware
from django.contrib.sessions.middleware import SessionMiddleware


@contextlib.contextmanager
def middleware(request):
    """Annotate a request object with a session"""
    middleware = SessionMiddleware()
    middleware.process_request(request)
    request.session.save()

    """Annotate a request object with a messages"""
    middleware = MessageMiddleware()
    middleware.process_request(request)
    request.session.save()
    yield request


class MockTransport(RequestsTransport):
    def __init__(self, return_values=None):
        self.return_values = {}
        self.input_values = {}
        if return_values:
            self.return_values.update(return_values)

    def get(self, url, headers=None):
        if url in self.return_values:
            val = self.return_values.get(url)
            if isinstance(val, Exception):
                raise val
            return val
        else:
            raise ValueError(f"Brak odpowiedzi URL zdefiniowanej dla {url}")

    def post(self, url, headers=None, body=None, delete=False):
        self.input_values[url] = {"headers": headers, "body": body, "delete": delete}
        return self.get(url, headers)



# -*- encoding: utf-8 -*-

from django.test import TestCase, TransactionTestCase
from django.test.client import Client, RequestFactory
from django.contrib import auth

User = auth.get_user_model()


class UserRequestFactory(RequestFactory):
    def __init__(self, user, *args, **kw):
        self.user = user
        super(UserRequestFactory, self).__init__(*args, **kw)

    def get(self, *args, **kw):
        req = super(UserRequestFactory, self).get(*args, **kw)
        req.user = self.user
        return req

    def post(self, *args, **kw):
        req = super(UserRequestFactory, self).post(*args, **kw)
        req.user = self.user
        return req


class WebTestCaseMixin:
    def setUp(self):
        self.client = Client()
        self.factory = RequestFactory()


class WebTestCase(WebTestCaseMixin, TestCase):
    pass


class WebTransactionTestCase(WebTestCaseMixin, TransactionTestCase):
    pass


class UserTestCaseMixin:
    USERNAME = "user"
    PASSWORD = "foo"
    EMAIL = "foo@bar.pl"
    create = User.objects.create_user

    def setUp(self):
        self.user = self.create(
            username=self.USERNAME, password=self.PASSWORD,
            email=self.EMAIL)
        res = self.client.login(username=self.USERNAME, password=self.PASSWORD)
        if res is not True:
            raise Exception("Cannot login")

        self.factory = UserRequestFactory(self.user)


class UserTestCase(UserTestCaseMixin, WebTestCase):
    def setUp(self):
        WebTestCase.setUp(self)
        UserTestCaseMixin.setUp(self)
    pass


class UserTransactionTestCase(UserTestCaseMixin, WebTransactionTestCase):
    pass

class SuperuserTestCase(UserTestCase):
    create = User.objects.create_superuser



# -*- coding: utf-8 -*-


from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0011_auto_20150212_2203'),
    ]

    operations = [
        migrations.AddField(
            model_name='wydzial',
            name='poprzednie_nazwy',
            field=models.CharField(default=b'', max_length=4096, null=True, blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='patent',
            name='dostep_dnia',
            field=models.DateField(help_text=b'Data dost\xc4\x99pu do strony WWW.', null=True, verbose_name=b'Dost\xc4\x99p dnia', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='praca_doktorska',
            name='dostep_dnia',
            field=models.DateField(help_text=b'Data dost\xc4\x99pu do strony WWW.', null=True, verbose_name=b'Dost\xc4\x99p dnia', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='praca_habilitacyjna',
            name='dostep_dnia',
            field=models.DateField(help_text=b'Data dost\xc4\x99pu do strony WWW.', null=True, verbose_name=b'Dost\xc4\x99p dnia', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='dostep_dnia',
            field=models.DateField(help_text=b'Data dost\xc4\x99pu do strony WWW.', null=True, verbose_name=b'Dost\xc4\x99p dnia', blank=True),
            preserve_default=True,
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='dostep_dnia',
            field=models.DateField(help_text=b'Data dost\xc4\x99pu do strony WWW.', null=True, verbose_name=b'Dost\xc4\x99p dnia', blank=True),
            preserve_default=True,
        ),
    ]



from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout
from crispy_forms_foundation.layout import ButtonHolder, Fieldset, Submit
from django import forms

from ewaluacja2021.models import ImportMaksymalnychSlotow, ZamowienieNaRaport


class ImportMaksymalnychSlotowForm(forms.ModelForm):
    class Meta:
        model = ImportMaksymalnychSlotow
        fields = ["plik"]

    def __init__(self, *args, **kw):
        self.helper = FormHelper()
        self.helper.form_class = "custom"
        self.helper.form_action = "."
        self.helper.layout = Layout(
            Fieldset(
                "Nowy import",
                "plik",
            ),
            ButtonHolder(
                Submit(
                    "submit",
                    "Dodaj",
                    css_id="id_submit",
                    css_class="submit button",
                ),
            ),
        )
        super().__init__(*args, **kw)


class ZamowienieNaRaportForm(forms.ModelForm):
    class Meta:
        model = ZamowienieNaRaport
        fields = ["dyscyplina_naukowa", "rodzaj"]

    def __init__(self, *args, **kw):
        self.helper = FormHelper()
        self.helper.form_class = "custom"
        self.helper.form_action = "."
        self.helper.layout = Layout(
            Fieldset(
                "Nowy raport 3N",
                "dyscyplina_naukowa",
                "rodzaj",
            ),
            ButtonHolder(
                Submit(
                    "submit",
                    "Zamów",
                    css_id="id_submit",
                    css_class="submit button",
                ),
            ),
        )
        super().__init__(*args, **kw)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.7 on 2017-08-23 22:26
from __future__ import unicode_literals

from decimal import Decimal
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0103_widoki_nowe_sumy'),
    ]

    operations = [
        migrations.CreateModel(
            name='Nowe_Sumy_View',
            fields=[
                ('impact_factor', models.DecimalField(db_index=True, decimal_places=3, default=Decimal('0.000'), max_digits=6)),
                ('punkty_kbn', models.DecimalField(db_index=True, decimal_places=2, default=Decimal('0.00'), max_digits=6, verbose_name='Punkty KBN')),
                ('index_copernicus', models.DecimalField(db_index=True, decimal_places=2, default=Decimal('0.00'), max_digits=6, verbose_name='Index Copernicus')),
                ('punktacja_wewnetrzna', models.DecimalField(db_index=True, decimal_places=2, default=Decimal('0.00'), max_digits=6, verbose_name='Punktacja wewnętrzna')),
                ('kc_impact_factor', models.DecimalField(blank=True, db_index=True, decimal_places=2, default=None, help_text='Jeżeli wpiszesz\n        wartość w to pole, to zostanie ona użyta w raporcie dla Komisji\n        Centralnej w punkcie IXa tego raportu.', max_digits=6, null=True, verbose_name='KC: Impact factor')),
                ('kc_punkty_kbn', models.DecimalField(blank=True, db_index=True, decimal_places=2, default=None, help_text='Jeżeli wpiszesz\n        wartość w to pole, to zostanie ona użyta w raporcie dla Komisji\n        Centralnej w punkcie IXa i IXb tego raportu.', max_digits=6, null=True, verbose_name='KC: Punkty KBN')),
                ('kc_index_copernicus', models.DecimalField(blank=True, decimal_places=2, default=None, help_text='Jeżeli wpiszesz\n        wartość w to pole, to zostanie ona użyta w raporcie dla Komisji\n        Centralnej w punkcie IXa i IXb tego raportu.', max_digits=6, null=True, verbose_name='KC: Index Copernicus')),
                ('weryfikacja_punktacji', models.BooleanField(default=False)),
                ('autor', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, primary_key=True, serialize=False, to='bpp.Autor')),
                ('jednostka', models.ForeignKey(on_delete=django.db.models.deletion.DO_NOTHING, primary_key=True, to='bpp.Jednostka')),
                ('rok', models.IntegerField()),
            ],
            options={
                'managed': False,
            },
        ),
    ]



# Generated by Django 3.0.14 on 2021-06-21 07:47

from django.db import migrations, models

import django.contrib.postgres.fields.jsonb


class Migration(migrations.Migration):

    dependencies = [
        ("pbn_api", "0011_auto_20210509_2339"),
    ]

    operations = [
        migrations.AddField(
            model_name="scientist",
            name="from_institution_api",
            field=models.NullBooleanField(db_index=True),
        ),
        migrations.AlterField(
            model_name="sentdata",
            name="data_sent",
            field=django.contrib.postgres.fields.jsonb.JSONField(
                verbose_name="Wysłane dane"
            ),
        ),
        migrations.AlterField(
            model_name="sentdata",
            name="exception",
            field=models.TextField(
                blank=True, max_length=65535, null=True, verbose_name="Kod błędu"
            ),
        ),
        migrations.AlterField(
            model_name="sentdata",
            name="last_updated_on",
            field=models.DateTimeField(auto_now=True, verbose_name="Data operacji"),
        ),
        migrations.AlterField(
            model_name="sentdata",
            name="uploaded_okay",
            field=models.BooleanField(
                db_index=True, default=True, verbose_name="Wysłano poprawnie"
            ),
        ),
    ]



import logging
from collections import defaultdict

from ewaluacja2021.const import LATA_2017_2018, LATA_2019_2021

logger = logging.getLogger(__name__)


class SumatorBase:
    def __init__(
        self, liczba_2_2_n, liczba_0_8_n, maks_pkt_aut_calosc, maks_pkt_aut_monografie
    ):
        self.liczba_2_2_n = liczba_2_2_n
        self.liczba_0_8_n = liczba_0_8_n
        self.maks_pkt_aut_calosc = maks_pkt_aut_calosc
        self.maks_pkt_aut_monografie = maks_pkt_aut_monografie
        self.liczba_2_2_n_minus_2 = self.liczba_2_2_n - 2
        self.liczba_0_8_n_minus_2 = self.liczba_0_8_n - 2

        self.zeruj()

    def czy_moze_przejsc_warunek_uczelnia(self, praca):
        if (
            self.sumy_slotow[LATA_2019_2021] < self.liczba_2_2_n_minus_2
            and self.sumy_slotow[LATA_2017_2018] < self.liczba_0_8_n_minus_2
        ):
            return True

        # Czy uczelnia nie ma już dość takich publikacji?
        if praca.rok >= 2019 or praca.monografia:
            if self.sumy_slotow[LATA_2019_2021] + praca.slot > self.liczba_2_2_n:
                return False
        else:
            if self.sumy_slotow[LATA_2017_2018] + praca.slot > self.liczba_0_8_n:
                return False

        # Uczelnia nie ma dość takich publikacji. Idziemy dalej.
        return True

    def czy_moze_przejsc_warunek_autor(self, praca):
        # Czy autor nie ma dość takich publikacji?
        if self.suma_prac_autorow_wszystko[
            praca.autor_id
        ] + praca.slot > self.maks_pkt_aut_calosc.get(praca.autor_id):
            return False

        # Jeżeli to jest monografia - czy autor nie ma dość już takich punktów za monografię?
        if praca.monografia:
            if self.suma_prac_autorow_monografie[
                praca.autor_id
            ] + praca.slot > self.maks_pkt_aut_monografie.get(praca.autor_id):
                return False

        return True

    def czy_moze_przejsc(self, praca):
        if (
            praca.id not in self.id_rekordow
            and self.czy_moze_przejsc_warunek_uczelnia(praca)
            and self.czy_moze_przejsc_warunek_autor(praca)
        ):
            return True

    def zsumuj_pojedyncza_prace(self, praca, indeks_solucji=None):
        self.suma_pkd += praca.pkdaut

        # Tu dodajemy Cache_Punktacja_Autora.id, nie zaś rekord_id
        self.id_rekordow.add(praca.id)
        if indeks_solucji is not None:
            self.indeksy_solucji.add(indeks_solucji)

        if praca.rok >= 2019 or praca.monografia:
            self.sumy_slotow[LATA_2019_2021] += praca.slot
        else:
            self.sumy_slotow[LATA_2017_2018] += praca.slot

        self.suma_prac_autorow_wszystko[praca.autor_id] += praca.slot
        if praca.monografia:
            self.suma_prac_autorow_monografie[praca.autor_id] += praca.slot

    def zeruj(self):
        self.id_rekordow = set()

        self.suma_pkd = 0
        self.sumy_slotow = [0, 0]
        self.indeksy_solucji = set()

        self.suma_prac_autorow_wszystko = defaultdict(int)
        self.suma_prac_autorow_monografie = defaultdict(int)



from .conference import Conference  # noqa
from .country import Country  # noqa
from .institution import (  # noqa
    Institution,
    OswiadczenieInstytucji,
    PublikacjaInstytucji,
)
from .journal import Journal  # noqa
from .language import Language  # noqa
from .publication import Publication  # noqa
from .publisher import Publisher  # noqa
from .scientist import Scientist  # noqa
from .sentdata import SentData  # noqa



# -*- encoding: utf-8 -*-


from django.contrib import admin

from bpp.admin.wydawnictwo_autor_base import Wydawnictwo_Autor_Base
from bpp.models import Wydawnictwo_Ciagle
from bpp.models.wydawnictwo_ciagle import Wydawnictwo_Ciagle_Autor


@admin.register(Wydawnictwo_Ciagle_Autor)
class Wydawnictwo_Ciagle_Autor_Admin(Wydawnictwo_Autor_Base):
    klasa_autora = Wydawnictwo_Ciagle_Autor
    base_rekord_class = Wydawnictwo_Ciagle
    change_list_template = "admin/bpp/wydawnictwo_ciagle_autor/change_list.html"



# Generated by Django 3.2.14 on 2022-07-08 11:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("tee", "0005_django32"),
    ]

    operations = [
        migrations.AlterField(
            model_name="log",
            name="finished_successfully",
            field=models.BooleanField(blank=True, default=None, null=True),
        ),
    ]



# Generated by Django 2.1.7 on 2019-03-03 07:42

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0152_merge'),
    ]

    operations = [
        migrations.AlterField(
            model_name='autor',
            name='aktualna_funkcja',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='aktualna_funkcja', to='bpp.Funkcja_Autora'),
        ),
        migrations.AlterField(
            model_name='autor',
            name='aktualna_jednostka',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='aktualna_jednostka', to='bpp.Jednostka'),
        ),
        migrations.AlterField(
            model_name='autor',
            name='plec',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='bpp.Plec'),
        ),
        migrations.AlterField(
            model_name='autor',
            name='tytul',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='bpp.Tytul'),
        ),
        migrations.AlterField(
            model_name='autor_dyscyplina',
            name='dyscyplina',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Dyscyplina_Naukowa'),
        ),
        migrations.AlterField(
            model_name='autor_jednostka',
            name='funkcja',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='bpp.Funkcja_Autora'),
        ),
        migrations.AlterField(
            model_name='bppuser',
            name='last_name',
            field=models.CharField(blank=True, max_length=150, verbose_name='last name'),
        ),
        migrations.AlterField(
            model_name='charakter_formalny',
            name='charakter_pbn',
            field=models.ForeignKey(blank=True, default=None, help_text='Wartość wybrana w tym polu zostanie użyta jako zawartość tagu <is>\n                                      w plikach eksportu do PBN', null=True, on_delete=django.db.models.deletion.PROTECT, to='bpp.Charakter_PBN', verbose_name='Charakter PBN'),
        ),
        migrations.AlterField(
            model_name='jednostka',
            name='uczelnia',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Uczelnia'),
        ),
        migrations.AlterField(
            model_name='jednostka',
            name='wydzial',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='bpp.Wydzial', verbose_name='Wydział'),
        ),
        migrations.AlterField(
            model_name='jednostka_wydzial',
            name='jednostka',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Jednostka'),
        ),
        migrations.AlterField(
            model_name='jednostka_wydzial',
            name='wydzial',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Wydzial'),
        ),
        migrations.AlterField(
            model_name='nagroda',
            name='content_type',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='contenttypes.ContentType'),
        ),
        migrations.AlterField(
            model_name='nagroda',
            name='organ_przyznajacy',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.OrganPrzyznajacyNagrody'),
        ),
        migrations.AlterField(
            model_name='patent',
            name='informacja_z',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='bpp.Zrodlo_Informacji'),
        ),
        migrations.AlterField(
            model_name='patent',
            name='rodzaj_prawa',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='bpp.Rodzaj_Prawa_Patentowego'),
        ),
        migrations.AlterField(
            model_name='patent',
            name='status_korekty',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Status_Korekty'),
        ),
        migrations.AlterField(
            model_name='patent_autor',
            name='autor',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Autor'),
        ),
        migrations.AlterField(
            model_name='patent_autor',
            name='jednostka',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Jednostka'),
        ),
        migrations.AlterField(
            model_name='patent_autor',
            name='typ_odpowiedzialnosci',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Typ_Odpowiedzialnosci', verbose_name='Typ odpowiedzialności'),
        ),
        migrations.AlterField(
            model_name='praca_doktorska',
            name='informacja_z',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='bpp.Zrodlo_Informacji'),
        ),
        migrations.AlterField(
            model_name='praca_doktorska',
            name='jednostka',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Jednostka'),
        ),
        migrations.AlterField(
            model_name='praca_doktorska',
            name='jezyk',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Jezyk', verbose_name='Język'),
        ),
        migrations.AlterField(
            model_name='praca_doktorska',
            name='promotor',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='promotor_doktoratu', to='bpp.Autor'),
        ),
        migrations.AlterField(
            model_name='praca_doktorska',
            name='status_korekty',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Status_Korekty'),
        ),
        migrations.AlterField(
            model_name='praca_doktorska',
            name='typ_kbn',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Typ_KBN', verbose_name='Typ KBN'),
        ),
        migrations.AlterField(
            model_name='praca_habilitacyjna',
            name='informacja_z',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='bpp.Zrodlo_Informacji'),
        ),
        migrations.AlterField(
            model_name='praca_habilitacyjna',
            name='jednostka',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Jednostka'),
        ),
        migrations.AlterField(
            model_name='praca_habilitacyjna',
            name='jezyk',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Jezyk', verbose_name='Język'),
        ),
        migrations.AlterField(
            model_name='praca_habilitacyjna',
            name='status_korekty',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Status_Korekty'),
        ),
        migrations.AlterField(
            model_name='praca_habilitacyjna',
            name='typ_kbn',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Typ_KBN', verbose_name='Typ KBN'),
        ),
        migrations.AlterField(
            model_name='publikacja_habilitacyjna',
            name='content_type',
            field=models.ForeignKey(limit_choices_to=models.Q(models.Q(('app_label', 'bpp'), ('model', 'wydawnictwo_ciagle')), models.Q(('app_label', 'bpp'), ('model', 'wydawnictwo_zwarte')), models.Q(('app_label', 'bpp'), ('model', 'patent')), _connector='OR'), on_delete=django.db.models.deletion.PROTECT, to='contenttypes.ContentType'),
        ),
        migrations.AlterField(
            model_name='publikacja_habilitacyjna',
            name='praca_habilitacyjna',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Praca_Habilitacyjna'),
        ),
        migrations.AlterField(
            model_name='redakcja_zrodla',
            name='redaktor',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Autor'),
        ),
        migrations.AlterField(
            model_name='typ_kbn',
            name='charakter_pbn',
            field=models.ForeignKey(blank=True, default=None, help_text='Wartość wybrana w tym polu zostanie użyta jako \n        fallback, tzn. jeżeli dla charakteru formalnego danego rekordu nie \n        określono odpowiedniego charakteru PBN, to zostanie użyta wartość \n        tego pola, o ile wybrana. ', null=True, on_delete=django.db.models.deletion.PROTECT, to='bpp.Charakter_PBN', verbose_name='Charakter PBN'),
        ),
        migrations.AlterField(
            model_name='uczelnia',
            name='obca_jednostka',
            field=models.ForeignKey(blank=True, help_text='\n    Jednostka skupiająca autorów nieindeksowanych, nie będących pracownikami uczelni. Procedury importujące\n    dane z zewnętrznych systemów informatycznych będą przypisywać do tej jednostki osoby, które zakończyły\n    pracę na uczelni. ', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='obca_jednostka', to='bpp.Jednostka'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='charakter_formalny',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Charakter_Formalny', verbose_name='Charakter formalny'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='informacja_z',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='bpp.Zrodlo_Informacji'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='jezyk',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Jezyk', verbose_name='Język'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='konferencja',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='bpp.Konferencja'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='openaccess_czas_publikacji',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='bpp.Czas_Udostepnienia_OpenAccess', verbose_name='OpenAccess: czas udostępnienia'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='openaccess_licencja',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='bpp.Licencja_OpenAccess', verbose_name='OpenAccess: licencja'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='openaccess_tryb_dostepu',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='bpp.Tryb_OpenAccess_Wydawnictwo_Ciagle', verbose_name='OpenAccess: tryb dostępu'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='openaccess_wersja_tekstu',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='bpp.Wersja_Tekstu_OpenAccess', verbose_name='OpenAccess: wersja tekstu'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='status_korekty',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Status_Korekty'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle',
            name='typ_kbn',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Typ_KBN', verbose_name='Typ KBN'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle_autor',
            name='autor',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Autor'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle_autor',
            name='jednostka',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Jednostka'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle_autor',
            name='typ_odpowiedzialnosci',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Typ_Odpowiedzialnosci', verbose_name='Typ odpowiedzialności'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle_zewnetrzna_baza_danych',
            name='baza',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Zewnetrzna_Baza_Danych'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_ciagle_zewnetrzna_baza_danych',
            name='rekord',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='zewnetrzna_baza_danych', to='bpp.Wydawnictwo_Ciagle'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='charakter_formalny',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Charakter_Formalny', verbose_name='Charakter formalny'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='informacja_z',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='bpp.Zrodlo_Informacji'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='jezyk',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Jezyk', verbose_name='Język'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='konferencja',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='bpp.Konferencja'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='openaccess_czas_publikacji',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='bpp.Czas_Udostepnienia_OpenAccess', verbose_name='OpenAccess: czas udostępnienia'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='openaccess_licencja',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='bpp.Licencja_OpenAccess', verbose_name='OpenAccess: licencja'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='openaccess_tryb_dostepu',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='bpp.Tryb_OpenAccess_Wydawnictwo_Zwarte'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='openaccess_wersja_tekstu',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='bpp.Wersja_Tekstu_OpenAccess', verbose_name='OpenAccess: wersja tekstu'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='seria_wydawnicza',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='bpp.Seria_Wydawnicza'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='status_korekty',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Status_Korekty'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='typ_kbn',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Typ_KBN', verbose_name='Typ KBN'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte',
            name='wydawnictwo_nadrzedne',
            field=models.ForeignKey(blank=True, help_text='Jeżeli dodajesz rozdział,\n        tu wybierz pracę, w ramach której dany rozdział występuje.', null=True, on_delete=django.db.models.deletion.PROTECT, related_name='wydawnictwa_powiazane_set', to='bpp.Wydawnictwo_Zwarte'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte_autor',
            name='autor',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Autor'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte_autor',
            name='jednostka',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Jednostka'),
        ),
        migrations.AlterField(
            model_name='wydawnictwo_zwarte_autor',
            name='typ_odpowiedzialnosci',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Typ_Odpowiedzialnosci', verbose_name='Typ odpowiedzialności'),
        ),
        migrations.AlterField(
            model_name='wydzial',
            name='uczelnia',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Uczelnia'),
        ),
        migrations.AlterField(
            model_name='zrodlo',
            name='jezyk',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='bpp.Jezyk'),
        ),
        migrations.AlterField(
            model_name='zrodlo',
            name='openaccess_licencja',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='bpp.Licencja_OpenAccess', verbose_name='OpenAccess: licencja'),
        ),
        migrations.AlterField(
            model_name='zrodlo',
            name='rodzaj',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='bpp.Rodzaj_Zrodla'),
        ),
        migrations.AlterField(
            model_name='zrodlo',
            name='zasieg',
            field=models.ForeignKey(blank=True, default=None, null=True, on_delete=django.db.models.deletion.SET_NULL, to='bpp.Zasieg_Zrodla'),
        ),
    ]



# Generated by Django 2.1.7 on 2019-03-24 17:26

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('import_dyscyplin', '0012_import_dyscyplin_row_dyscyplina_ostateczna'),
    ]

    operations = [
        migrations.CreateModel(
            name='Kolumna',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('nazwa_w_pliku', models.CharField(help_text='Nazwa kolumny po stronie pliku importu', max_length=250)),
                ('rodzaj_pola', models.CharField(choices=[('nazwisko', 'nazwisko'), ('imię', 'imię'), ('pesel', 'pesel'), ('orcid', 'orcid'), ('nazwa jednostki', 'nazwa jednostki'), ('wydział', 'wydział'), ('dyscyplina', 'dyscyplina'), ('kod dyscypliny', 'kod dyscypliny'), ('procent dyscypliny', 'procent dyscypliny'), ('subdyscyplina', 'subdyscyplina'), ('kod subdyscypliny', 'kod subdyscypliny'), ('procent subdyscypliny', 'procent subdyscypliny')], max_length=50)),
                ('parent', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='import_dyscyplin.Import_Dyscyplin')),
            ],
        ),
        migrations.RemoveField(
            model_name='import_dyscyplin_row',
            name='dyscyplina_ostateczna',
        ),
        migrations.AddField(
            model_name='import_dyscyplin_row',
            name='procent_dyscypliny',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True),
        ),
        migrations.AddField(
            model_name='import_dyscyplin_row',
            name='procent_subdyscypliny',
            field=models.DecimalField(blank=True, decimal_places=2, max_digits=5, null=True),
        ),
        migrations.AddField(
            model_name='import_dyscyplin',
            name='kolumny',
            field=models.ManyToManyField(to='import_dyscyplin.Kolumna'),
        ),
    ]



"""Klasy pomocnicze dla klas ModelAdmin które chcą korzystać z parametru
GET ``numer_zgloszenia`` czyli wypełniać dane wstępne wg zawartości zgłoszenia
o danym numerze ID. """
from django.utils import timezone

from bpp import const
from bpp.admin.helpers import MODEL_Z_OPLATA_ZA_PUBLIKACJE


class KorzystaZNumeruZgloszeniaMixin:
    """Mixin dla klas korzystajacych z parametru 'numer_zgloszenia', który to numer
    powoduje wypełnienie wstępnych parametrów wg zgłoszenia o ID podanym jako parametr GET.
    """

    def get_zgloszenie_publikacji(self, request):
        if request.method == "GET":
            nz = request.GET.get(const.NUMER_ZGLOSZENIA_PARAM, "")
            if nz.isnumeric():
                from zglos_publikacje.models import Zgloszenie_Publikacji

                try:
                    return Zgloszenie_Publikacji.objects.get(pk=nz)
                except Zgloszenie_Publikacji.DoesNotExist:
                    pass


class KorzystaZNumeruZgloszeniaInlineMixin(KorzystaZNumeruZgloszeniaMixin):
    """Mixin dla klas Inline znajdujacych się w formualrzach ModelAdmin,
    korzystajacych ze zgłoszeń prac przez uzytkownikow. Funkcja ``get_extra`` zwraca
    ilosc formularzy, jaka powinna byc wyswietlona aby pokryc zapotrzebowanie na
    inicjalne wartosci dla autorów."""

    def get_extra(self, request, obj=None, **kwargs):
        z = self.get_zgloszenie_publikacji(request)
        if z is not None:
            return z.zgloszenie_publikacji_autor_set.count()
        return super().get_extra(request=request, obj=obj, **kwargs)


class UzupelniajWstepneDanePoNumerzeZgloszeniaMixin(
    KorzystaZNumeruZgloszeniaMixin,
):
    """Mixin dla klas ModelAdmin które chcą miec wypełniane parametry wg zgłoszenia
    z parametru requestu GET ``numer_zgloszenia``.

    Obecnie dla Wydawnictwo_Ciagle i Wydawnictwo_Zwarte.

    """

    def get_changeform_initial_data(self, request):
        z = self.get_zgloszenie_publikacji(request)
        if z is not None:
            ret = {
                "tytul_oryginalny": z.tytul_oryginalny,
                "rok": z.rok,
                "public_www": z.strona_www,
                "public_dostep_dnia": timezone.now().date(),
            }

            for pole in MODEL_Z_OPLATA_ZA_PUBLIKACJE:
                ret[pole] = getattr(z, pole)

            ret["adnotacje"] = (
                f"E-mail zgłaszającego: <{z.email}>.\nNumer zgłoszenia: {z.id} -- {str(z)}\n"
                f"Pole 'Dostęp dnia' ustawione automatycznie na datę utworzenia rekordu. "
            )

            return ret

        return super().get_changeform_initial_data(request)

    def get_formset_kwargs(self, request, obj, inline, prefix):
        if not (
            prefix == "autorzy_set"
            and obj.pk is None
            and str(type(inline)).find("<locals>.baseModel_AutorInline") > -1
        ):
            return super().get_formset_kwargs(request, obj, inline, prefix)

        z = self.get_zgloszenie_publikacji(request)

        if z is None:
            return super().get_formset_kwargs(request, obj, inline, prefix)

        initial = []
        for zpa in z.zgloszenie_publikacji_autor_set.all():
            initial.append(
                {
                    "autor": zpa.autor_id,
                    "jednostka": zpa.jednostka_id,
                    "dyscyplina_naukowa": zpa.dyscyplina_naukowa_id,
                }
            )

        return {
            "initial": initial,
        }



# Generated by Django 2.2.8 on 2020-02-13 20:48

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("bpp", "0194_auto_20200213_2148"),
    ]

    operations = [
        migrations.CreateModel(
            name="RaportZerowyEntry",
            fields=[
                (
                    "autor",
                    models.OneToOneField(
                        on_delete=django.db.models.deletion.DO_NOTHING,
                        primary_key=True,
                        serialize=False,
                        to="bpp.Autor",
                    ),
                ),
                ("rok", models.IntegerField()),
            ],
            options={"ordering": ("autor", "rok"), "managed": False,},
        ),
    ]



from pbn_api.admin.base import BaseMongoDBAdmin
from pbn_api.admin.filters import OdpowiednikJednostkiWBPPFilter
from pbn_api.models import Institution

from django.contrib import admin


@admin.register(Institution)
class InstitutionAdmin(BaseMongoDBAdmin):

    list_display = [
        "name",
        "addressCity",
        "addressStreet",
        "addressStreetNumber",
        "addressPostalCode",
        "website",
        "rekord_w_bpp",
    ]

    search_fields = [
        "mongoId",
        "name",
        "addressCity",
        "addressStreet",
        "addressPostalCode",
        "polonUid",
    ]
    list_filter = [OdpowiednikJednostkiWBPPFilter] + BaseMongoDBAdmin.list_filter



# -*- coding: utf-8 -*-

import os
from django.db import models, migrations
import csv

def migration_open(fn):
    p = os.path.join(
        os.path.dirname(__file__),
        fn)

    return open(p, encoding="utf-8")

def dodaj_dane_typow_pbn(apps, schema_editor):
    Charakter_PBN = apps.get_model("bpp", "Charakter_PBN")

    mapa = dict()
    for row in csv.reader(migration_open("0017_typy_pbn_map.txt")):
        for elem in row[1:]:
            mapa[elem] = row[0]

    for identyfikator, opis, help_text in csv.reader(
            migration_open("0017_typy_pbn.txt"), delimiter=';'):
        Charakter_PBN.objects.create(
            wlasciwy_dla=mapa.get(identyfikator, "article"),
            identyfikator=identyfikator,
            opis=opis,
            help_text=help_text)

class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0016_auto_20150824_1051'),
    ]

    operations = [
        migrations.CreateModel(
            name='Charakter_PBN',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('wlasciwy_dla', models.CharField(max_length=20, choices=[
                    (b'article', b'Artyku\xc5\x82'),
                    (b'book', b'Ksi\xc4\x85\xc5\xbcka'),
                    (b'chapter', b'Rozdzia\xc5\x82')])),
                ('identyfikator', models.CharField(max_length=100)),
                ('opis', models.CharField(max_length=500)),
                ('help_text', models.TextField(blank=True)),
            ],
            options={
                'ordering': ['identyfikator'],
            },
            bases=(models.Model,),
        ),
        migrations.RunPython(dodaj_dane_typow_pbn)
    ]



# Generated by Django 3.0.14 on 2021-05-29 21:51

from django.db import migrations


def wyzeruj_zrodla_pbn_uid_id(apps, schema_editor):
    Zrodlo = apps.get_model("bpp", "Zrodlo")
    Zrodlo.objects.exclude(pbn_uid_id=None).update(pbn_uid_id=None)


class Migration(migrations.Migration):

    dependencies = [
        ("bpp", "0268_auto_20210519_0238"),
    ]

    operations = [
        migrations.RunPython(wyzeruj_zrodla_pbn_uid_id, migrations.RunPython.noop),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2018-04-14 15:27
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import mptt.fields


class Migration(migrations.Migration):

    dependencies = [
        ('bpp', '0121_rekord_ostatnio_zmieniony_dla'),
    ]

    operations = [
        migrations.CreateModel(
            name='Autor_Dyscyplina',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('rok', models.PositiveSmallIntegerField()),
                ('autor', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bpp.Autor')),
            ],
        ),
        migrations.CreateModel(
            name='Dyscyplina_Naukowa',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('nazwa', models.CharField(max_length=200)),
                ('widoczna', models.BooleanField(default=True)),
                ('lft', models.PositiveIntegerField(db_index=True, editable=False)),
                ('rght', models.PositiveIntegerField(db_index=True, editable=False)),
                ('tree_id', models.PositiveIntegerField(db_index=True, editable=False)),
                ('level', models.PositiveIntegerField(db_index=True, editable=False)),
                ('dyscyplina_nadrzedna', mptt.fields.TreeForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='subdyscypliny', to='bpp.Dyscyplina_Naukowa')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='autor_dyscyplina',
            name='dyscyplina',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='bpp.Dyscyplina_Naukowa'),
        ),
    ]



import time
import warnings


def wait_for_object(klass, pk, no_tries=10):
    warnings.warn(
        "Ta funkcja niepotrzebnie 'przetrzymuje' workera przez 10 sekund. "
        "Rozsadne byloby jej nie uzywac i przepisac kod na konstrukcje zblizona do"
        "long_running.tasks.perform_generic_long_running_task -- czyli funkcje, ktora"
        "probuje uruchomic sie za 10 sekund za pomoca mechanizmow celery, nie zas "
        "blokujaca worker za pomoca time.sleep ... ",
        category=DeprecationWarning,
    )

    obj = None

    while no_tries > 0:
        try:
            obj = klass.objects.get(pk=pk)
            break
        except klass.DoesNotExist:
            time.sleep(1)
            no_tries = no_tries - 1

    if obj is None:
        raise klass.DoesNotExist("Cannot fetch klass %r with pk %r" % (klass, pk))

    return obj



# -*- encoding: utf-8 -*-
import os

from celery.utils.log import get_task_logger
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.core.management import call_command

try:
    from django.core.urlresolvers import reverse
except ImportError:
    from django.urls import reverse

from celeryui.interfaces import IWebTask
from celeryui.models import Report

from bpp.models import (
    Praca_Doktorska,
    Praca_Habilitacyjna,
    Uczelnia,
    Wydawnictwo_Ciagle,
    Wydawnictwo_Zwarte,
)
from bpp.util import remove_old_objects

logger = get_task_logger(__name__)

from django_bpp.celery_tasks import app


@app.task(ignore_result=True)
def remove_file(path):
    if path.startswith(os.path.join(settings.MEDIA_ROOT, "report")):
        logger.warning("Removing %r" % path)
        os.unlink(path)


@app.task
def make_report(uid):
    from celeryui.models import Report

    from bpp import reports  # for registry

    reports  # pycharm, don't clean this plz

    report = Report.objects.get(uid=uid)
    report.started()

    def execute():
        report.execute(raise_exceptions=True)
        msg = 'Ukończono generowanie raportu "%s", <a href="%s">kliknij tutaj, aby otworzyć</a>. '
        url = reverse("bpp:podglad-raportu", args=(report.uid,))
        call_command(
            "send_message",
            report.ordered_by.username,
            msg % (IWebTask(report).title, url),
        )

    return execute()


task_limits = {}


def my_limit(fun):
    res = task_limits.get(fun)
    if not res or (res.successful() or res.failed()):
        task_limits[fun] = fun.apply_async(
            countdown=settings.MAT_VIEW_REFRESH_COUNTDOWN
        )
        return

    if res:
        logger.info("Task %r has been revoked." % res.id)
        res.revoke()
        task_limits[fun] = fun.apply_async(
            countdown=settings.MAT_VIEW_REFRESH_COUNTDOWN
        )


@app.task(ignore_result=True, store_errors_even_if_ignored=True)
def remove_old_report_files():
    return remove_old_objects(Report, field_name="started_on")


def _zaktualizuj_liczbe_cytowan(klasy=None):
    if klasy is None:
        klasy = (
            Wydawnictwo_Ciagle,
            Wydawnictwo_Zwarte,
            Praca_Doktorska,
            Praca_Habilitacyjna,
        )

    for uczelnia in Uczelnia.objects.all():
        try:
            client = uczelnia.wosclient()
        except ImproperlyConfigured:
            continue

        # FIXME: jeżeli jest >1 uczelnia w systemie, to odpytanie
        # obiektów nastąpi w sposób wielokrotny...

        for klass in klasy:
            filtered = (
                klass.objects.all()
                .exclude(doi=None)
                .exclude(pubmed_id=None)
                .values("id", "doi", "pubmed_id")
            )

            for grp in client.query_multiple(filtered):
                for k, item in grp.items():
                    changed = False

                    timesCited = item.get("timesCited")
                    doi = item.get("doi")
                    pubmed_id = item.get("pmid")

                    obj = klass.objects.get(pk=k)

                    if timesCited is not None:
                        if obj.liczba_cytowan != timesCited:
                            obj.liczba_cytowan = timesCited
                            changed = True

                    if pubmed_id is not None:
                        if obj.pubmed_id != pubmed_id:
                            obj.pubmed_id = pubmed_id
                            changed = True

                    if doi is not None:
                        if obj.doi != doi:
                            obj.doi = doi
                            changed = True

                    if changed:
                        obj.save()


@app.task
def zaktualizuj_liczbe_cytowan():
    _zaktualizuj_liczbe_cytowan()
